// -*- C++ -*-
/*!
 * @file  arm.h
 * @brief Arm
 * @date  $Date$
 *
 * $Id$
 */

#ifndef ARM_H
#define ARM_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include "Palib.h"
#include "Constant.h"
#include "linux/art_task.h"
#define ARMNO ARM0

using namespace RTC;

class arm
  : public RTC::DataFlowComponentBase
{
 public:
  arm(RTC::Manager* manager);
  ~arm();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);


 protected:
  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedFloatSeq m_ReferenceVelocity;
  InPort<TimedFloatSeq> m_ReferenceVelocityIn;
  
  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedFloatSeq m_CurrentPosition;
  OutPort<TimedFloatSeq> m_CurrentPositionOut;
  
  Palib* m_palib;

 private:
  int dummy;

};


extern "C"
{
  void armInit(RTC::Manager* manager);
};

#endif // ARM_H
