// -*- C++ -*-
/*!
 * @file  controllerDamper.h
 * @brief controller
 * @date  $Date$
 *
 * $Id$
 */

#ifndef CONTROLLERDAMPER_H
#define CONTROLLERDAMPER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include "linux/art_task.h"

using namespace RTC;

class controllerDamper
  : public RTC::DataFlowComponentBase
{
 public:
  controllerDamper(RTC::Manager* manager);
  ~controllerDamper();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  virtual RTC::ReturnCode_t onInitialize();

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);


 protected:
  // DataInPort declaration
  TimedFloatSeq m_ReferenceForce;
  InPort<TimedFloatSeq> m_ReferenceForceIn;
  TimedFloatSeq m_CurrentForce;
  InPort<TimedFloatSeq> m_CurrentForceIn;
  TimedFloatSeq m_DamperGain;
  InPort<TimedFloatSeq> m_DamperGainIn;

  // DataOutPort declaration
  TimedFloatSeq m_ReferenceVelocity;
  OutPort<TimedFloatSeq> m_ReferenceVelocityOut;
  
  TimedFloatSeq m_thisCurrentForce;
  TimedFloatSeq m_thisDamperGain;
  TimedFloatSeq m_thisReferenceForce;

 private:
  int dummy;

};


extern "C"
{
  void controllerDamperInit(RTC::Manager* manager);
};

#endif // CONTROLLERDAMPER_H
