#include "PoolData.h"

using namespace std;

PoolData::PoolData()
  :size(5000), count(0), full(false), output_filename("default_output.txt")
{
  data.resize(size);
}

void PoolData::SetConf(const char* capture, int dataNumber)
{
  strcpy(m_capture, capture);
  m_dataNumber = dataNumber;
}

void PoolData::SetDataSize(long num)
{
  size = num;
  data.resize(size);
  Reset();
}

void PoolData::Output(void)
{
  ofstream ofile(output_filename.c_str(), ios::app);
//   ofile << "\"CurrentTime(sec)\",\"SentTime(sec)\",\"data1\",\"data2\",\"data3\",\"data4\",\"data5\",\"data6\"" << endl;
  ofile << m_capture << endl;
  for (unsigned int i = 0; i < data.size(); i++) {
    ofile << data[i]; 
  }
  ofile << endl;
}

void PoolData::Reset(void)
{
  count = 0;
  full = false;
  data.clear();
}

void PoolData::SetData(unsigned long sec, unsigned long nsec, RTC::TimedFloatSeq val)
{
  static unsigned long sec0, nsec0;
  static unsigned long oldSec, oldNsec;  

  if (!full && count == 0) {
    sec0 = sec;
    nsec0 = nsec;
    oldSec = sec;
    oldNsec = nsec;
  }

  if (count > 0) {
    if (sec == oldSec && nsec == oldNsec) {
      return;
    }
  }

  oldSec = sec;
  oldNsec = nsec;

  sec -= sec0;
  if (nsec > nsec0) {
    nsec -= nsec0;
  } else {
    nsec += ~nsec0;
    sec -= 1;
  }

  data.push_back(createLine(sec, nsec, val));
  
  if (!full) {
    count++;
    if (count >= size)
      full = true;
  } else {
    data.pop_front();
  }
}

string PoolData::createLine(unsigned long sec, unsigned long nsec, RTC::TimedFloatSeq val)
{
  ostringstream data;
//   data << sec << '.';
//   data.fill('0');
//   data.width(9);
//   data << nsec << ',';

  data << val.tm.sec << '.';
  data.fill('0'); 
  data.width(9);
  data << val.tm.nsec;
  for (int i = 0; i < val.data.length(); i++)
    data << ',' << val.data[i];
  data << std::endl;

  return data.str();
}
