// -*- C++ -*-
/*!
 * @file  dataLogger.cpp
 * @brief Data Logger
 * $Date$
 *
 * $Id$
 */

#include "dataLogger.h"

#define TS_COUNT 1000
#define PERIOD 2000
#define OUTPUT_LIMIT 5000
//#define FLAG_WAIT_COUNT 300
#include "../common/timestat.h"
// #define VERSION "1.04 (050927)"

//#define ART

#ifdef ART
#include "linux/art_task.h"
#endif

using namespace std;
int readConfiguration(GraphProperty& gp, const string fname);

// Module specification
static const char* datalogger_spec[] =
  {
    "implementation_id", "dataLogger",
    "type_name",         "DataFlowComponent",
    "description",       "Data Logger",
    "version",           "1.0",
    "vendor",            "K.Kitagaki (A shift to version 0.4.0 by S.Kurihara)",
    "category",          "graph",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };


dataLogger::dataLogger(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    m_DisplayDataIn("DisplayData", m_DisplayData),
    dummy(0)
{
  if (readConfiguration(graphProperty, configurationFileName) != 0)
    ::exit(1);
   char line[128];
   sprintf(line, "/DataLogger/%s|rtc", graphProperty.title.c_str());

  // Registration: InPort/OutPort/Service
  // Set InPort buffers
  registerInPort("DisplayData", m_DisplayDataIn);
}

dataLogger::~dataLogger()
{
}


RTC::ReturnCode_t dataLogger::onInitialize()
{
  //  cout << "Datalogger ver." << VERSION << " T.Sota, K.Kitagaki" << endl;

  char capture[512];
  capture[0] = '\0';

  strcat(capture, "\"Time\",");
  for (int j = 0; j < graphProperty.data.size(); j++) {
    strcat(capture, "\"");
    strcat(capture, graphProperty.data[j].capture.c_str());
    strcat(capture, "\",");
  }
  capture[strlen(capture)-1] = '\0';

  m_pdata.SetConf(capture, graphProperty.data.size());
  // for datalogger extention
  m_pool_data = true; // data pool flag. normally true

#ifdef ART
  m_change_period = false;
#endif
  m_period = (int)(graphProperty.loggingCycle * 1e6);

  m_display_out = true;
  m_display_int_count = 1.0;  // sec

  // for graph initialize
  m_gw = new GraphWindow(graphProperty.title.c_str(), 
		graphProperty.windowWidth, graphProperty.windowHeight);
  m_gw->SetTimescale(graphProperty.timeScale);

  for (int i = 0; i < graphProperty.graphNumber; i++) {
    capture[0] = '\0';
    for (int j = 0; j < graphProperty.data.size(); j++) {
      if (graphProperty.data[j].graphId == i) {
	strcat(capture, graphProperty.data[j].capture.c_str());
	strcat(capture, ", ");
      }
    }
    sprintf(&capture[strlen(capture)], "[%f %f]", graphProperty.graph[i].min, graphProperty.graph[i].max);
    //    capture[strlen(capture)-2] = '\0';
    m_gw->SetGraphParameters(i, capture); 
  }

  for (int i = 0; i < graphProperty.data.size(); i++) {
    m_gw->SetRange2(i,
		   graphProperty.graph[graphProperty.data[i].graphId].min, 
		   graphProperty.graph[graphProperty.data[i].graphId].max,
		   graphProperty.data[i].graphId);
  }
//   m_gw->SetForceRange(20.0);
//   m_gw->SetMomentRange(2.0);

  m_graph_out = true;
  m_graph_int_count = .02;  // sec
//    m_graph_counter = 0;

  m_inp = new Interp(this);

  // end of datalogger extention
  return RTC::OK;
}


RTC::ReturnCode_t dataLogger::onFinalize()
{
  delete m_inp;
  delete m_gw;

  return RTC::OK;
}


RTC::ReturnCode_t dataLogger::onActivated(RTC::UniqueId ec_id)
{
  m_gw->Reset();
#ifdef ART
  if (art_enter(ART_PRIO_MAX, ART_TASK_PERIODIC, m_period) == -1) {
      cerr << "abort: art_enter" << endl;
      return RTC::ERROR;
    } else {
      cerr << "art_enter : OK." << endl;
    }
#endif
  m_prev_disp_time = m_prev_graph_time = 0.;
  m_loop = true;
  m_do_first = true;

  cout << "ON Activate." << endl;
  return RTC::OK;
}



RTC::ReturnCode_t dataLogger::onExecute(RTC::UniqueId ec_id)
{
  
#ifdef ART
  if (art_wait() == -1) {
    cerr << "abort: art_wait" << endl;
    return RTC::ERROR;
  }
#endif
#ifdef ART
  if (m_change_period) {
    if (art_exit() == -1) {
      cerr << "abort: art_exit" << endl;
      return RTC::ERROR;
    }
	  
    if (art_enter(ART_PRIO_MAX, ART_TASK_PERIODIC, m_period) == -1) {
      cerr << "abort: art_enter" << endl;
      return RTC::ERROR;
    }
    m_change_period = false;
  } 
#endif

  m_DisplayData = m_DisplayDataIn.read();
  timeval tv;
  gettimeofday(&tv, 0);
  
  cout << "m_DisplayData.data.length(): " << m_DisplayData.data.length() << endl;
  if (m_DisplayData.data.length() > 0) {
    //// pool data
    if (m_pool_data) {
      m_pdata.SetData(tv.tv_sec, (1e+3 * tv.tv_usec), 
      		      m_DisplayData);
      for (int i = 0; i < m_DisplayData.data.length(); i++) {
	cout << "data[" << i << "] = " << m_DisplayData.data[i] << endl;
      }
    }
    if (m_gw->IsReqestFileOutput())
      m_pdata.Output();

    double timeg = tv.tv_sec + 1e-6 * tv.tv_usec;
    if (m_display_out) {
      if (m_prev_disp_time + m_display_int_count < timeg 
	  || m_prev_disp_time > timeg) {
	printf("%16.6f [s]: ", timeg);
	for (int i = 0; i < m_DisplayData.data.length(); i++) 
	  cout << m_DisplayData.data[i] << ", ";
	cout << endl;
	m_prev_disp_time = timeg;
      }
    }

    if (m_graph_out) {
      if (m_prev_graph_time + m_graph_int_count < timeg || m_prev_graph_time > timeg)
	{
	  double time = (double)tv.tv_usec * 1.0e-6
	    + (double)tv.tv_sec;
	  float data[128];
	  // cout << m_DisplayData.data.length() << "]" << endl;
	  for (int i = 0; i < m_DisplayData.data.length(); i++)
	    data[i] = m_DisplayData.data[i];
	  m_gw->SetData(time, data);
	  if (!m_do_first) {
	    m_gw->Draw();
	  } else {
	    m_do_first = false;
	  }
	  m_prev_graph_time = timeg;
	}
    }
  }

  return RTC::OK;
}


RTC::ReturnCode_t dataLogger::onDeactivated(RTC::UniqueId ec_id)
{
#ifdef ART
  if (art_exit() == -1) {
    cerr << "abort: art_exit" << endl;
    return RTC::ERROR;
  }
#endif

  return RTC::OK;
}


void dataLogger::quit()
{
  m_loop = false;
}

void dataLogger::setRange(CORBA::Long portNumber, 
			  CORBA::Float min, CORBA::Float max)
{
  m_gw->SetRange(portNumber, min, max);
}

#ifdef ART
void dataLogger::setPeriod(CORBA::Long usec)
{
  m_period = usec;
  m_change_period = true;
}
#endif

CORBA::Long dataLogger::putData()
{
  return 0;
}

CORBA::Long dataLogger::setLogData(const char* fname, CORBA::Long num)
{
  if (!fname)
    return -1;

  if (num <= 0 || num > OUTPUT_LIMIT)
    return OUTPUT_LIMIT;
  

  m_pool_data = false;

  m_pdata.Reset();
  m_pdata.SetOutputFilename(fname);
  m_pdata.SetDataSize(num);

  cout << "m_pool_data true" << endl;
  m_pool_data = true;
  cout << "m_pool_data true" << endl;
  return 0;
}


void dataLogger::displayOutput(CORBA::Boolean flag, CORBA::Long intval)
{
  m_display_int_count = intval/10.;
  m_display_out = flag;
}


int readConfiguration(GraphProperty& gp, const string fname) {
  ifstream fs;
  fs.open(fname.c_str());
  if (fs.bad()) {
    cout << "*** Error: File open error [" << fname.c_str() << "]" << endl;
    exit(1);
  }

  char line[128];
  char* str_prt;

  char id[128];
  fs.getline(line, 128); // pass ID
  sscanf(line, "%s", id);
  if (strcmp(id, "DataLogger") != 0) {
    cout << "*** Error: configuration file[" << fname.c_str() << "] is not for dataLogger" << endl;
    exit(1);
  }

  char title[128];
  fs.getline(line, 128);
  sscanf(line, "%s", title);
  gp.title = title;

  char scaleType[128];
  fs.getline(line, 128);
  sscanf(line, "%s", scaleType);
  if (strcmp(scaleType, "log") == 0) {
    gp.scaleType = SCALE_LOG;
  } else if (strcmp(scaleType, "linear") == 0) {
    gp.scaleType = SCALE_LINEAR;
  } else {
    cout << "*** Error: Configuration file description invalid: Scale type must be 'linear' or 'log'." << endl;
    exit(1);
  }

  fs.getline(line, 128);
  sscanf(line, "%f", &gp.loggingCycle);

  fs.getline(line, 128);
  sscanf(line, "%f", &gp.timeScale);

  fs.getline(line, 128);
  sscanf(line, "%d %d", &gp.windowWidth, &gp.windowHeight);

  fs.getline(line, 128);
  sscanf(line, "%d", &gp.graphNumber);

  MinMax graph;
  for (int i = 0; i < gp.graphNumber; i++) { 
    fs.getline(line, 128);
    sscanf(line, "%f %f", &graph.min, &graph.max);
    gp.graph.push_back(graph);
  }

  DataProperty data;
  char capture[128];
  while (1) {
    fs.getline(line, 128);
    if (fs.eof())
      break;
    if (sscanf(line, "%d %s", 
	       &data.graphId, capture) == EOF)
      break;
    data.capture = capture;
    gp.data.push_back(data);
  }
  return 0;
}


extern "C"
{
 
  void dataLoggerInit(RTC::Manager* manager)
  {
    RTC::Properties profile(datalogger_spec);
    manager->registerFactory(profile,
                             RTC::Create<dataLogger>,
                             RTC::Delete<dataLogger>);
  }
  
};


