// -*- C++ -*-
/*!
 * @file  dataLogger.h
 * @brief Data Logger
 * @date  $Date$
 *
 * $Id$
 */

#ifndef DATALOGGER_H
#define DATALOGGER_H

#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <fstream>

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include "PoolData.h"
#include "GraphWindow.h"
class Interp;
#include "Interp.h"

using namespace RTC;

class dataLogger
  : public RTC::DataFlowComponentBase
{
 public:
  dataLogger(RTC::Manager* manager);
  ~dataLogger();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  void quit(void);

#ifdef ART
  void setPeriod(CORBA::Long usec);
#endif
  void setRange(CORBA::Long portNumber, CORBA::Float min, CORBA::Float max);
  CORBA::Long fileOutput(const char* filename, CORBA::Long num);
  CORBA::Long setLogData(const char* filename, CORBA::Long num);
  CORBA::Long putData();
  void displayOutput(CORBA::Boolean flag, CORBA::Long intval);

 private:
  PoolData m_pdata;    // data pool
  void initFileOutput(int num);

  // datalogger extention
  bool m_pool_data;               // data pool flag. normally true 

#ifdef ART  
  bool m_change_period;           // flag for change period
#endif
  int  m_period;                  // ART period

//    int  display_int_count;       // interval count for display output
  double m_display_int_count;       // interval time for display output
  bool m_display_out;             // flag for display output
  int  m_display_counter;         // counter for display output
  double m_prev_disp_time;

  Interp* m_inp;
  GraphWindow* m_gw;              // pointer to GraphWindow object
  //  int  m_graph_int_count;         // interval count for graph output
  double m_graph_int_count;         // interval time for graph output(sec)
  bool m_graph_out;               // flag for graph output
  int  m_graph_counter;           // counter for graph output
  double m_prev_graph_time;

  bool m_loop;                    // main loop flag

#ifdef INTERP
  void interpretLoop(void);
  void printUsages(void);
#endif

  bool m_do_first;

  // end of datalogger extention  


 protected:
  // DataInPort declaration
  TimedFloatSeq m_DisplayData;
  InPort<TimedFloatSeq> m_DisplayDataIn;
  

 private:
  int dummy;

};


struct MinMax{
  float min;
  float max;
};
struct DataProperty{
  int graphId;
  string capture;
};

enum ScaleType {
  SCALE_LINEAR,
  SCALE_LOG
};

struct GraphProperty {
  string title;
  ScaleType scaleType;
  float loggingCycle;
  float timeScale;
  int windowWidth;
  int windowHeight;
  int graphNumber;
  vector<MinMax> graph;
  vector<DataProperty> data;
};

extern GraphProperty graphProperty;
extern string configurationFileName;

extern "C"
{
  void dataLoggerInit(RTC::Manager* manager);
};

#endif // DATALOGGER_H
