// -*- C++ -*-
/*!
 * @file  ftsensor.h
 * @brief F/T Sensor
 * @date  $Date$
 *
 * $Id$
 */

#ifndef FTSENSOR_H
#define FTSENSOR_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include "NittaFts.h"
#include "linux/art_task.h"

#undef OK
#undef ERROR

#define SENSOR_NO 1

using namespace RTC;

class ftsensor
  : public RTC::DataFlowComponentBase
{
 public:
  ftsensor(RTC::Manager* manager);
  ~ftsensor();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);


 protected:
  // DataOutPort declaration
  TimedFloatSeq m_Force;
  OutPort<TimedFloatSeq> m_ForceOut;
  
  NittaFts* m_nittaFts;

 private:
  int dummy;

};


extern "C"
{
  void ftsensorInit(RTC::Manager* manager);
};

#endif // FTSENSOR_H
