#ifndef _angle_limitter_impl_h
#define _angle_limitter_impl_h

//#include "patype.h"
#include "Pa10Controller.h"
#include "Pa10.h"
#include "PalibController.h"
#include "Constant.h"
#include "AngleLimitter.h"

#undef A3D
#define	A3D	D2R(-130) // originally -130
#undef A3U
#define A3U	D2R(130) // originally 130


enum JointLimitState {JointLimitMinOver = -1, JointLimitOK = 0, JointLimitMaxOver = 1};
const float fHardAngMax[7] = {A0U, A1U, A2U, A3U, A4U, A5U, A6U};	// cannot set max angle above this
const float fHardAngMin[7] = {A0D, A1D, A2D, A3D, A4D, A5D, A6D};	// cannot set min angle below this

//class Pa10;

class AngleLimitterImpl : public AngleLimitter {
protected:
    Pa10Controller * m_pController;		// pointer to the Pa10 managed by this
    float m_fCurrAngle[7];	// keep current angle here
    float m_fAngMemo[2][7];	// keep SAFE joint angles on the latest path, [0] is last, [1] is second last
    int m_iCurrMemoNum;		// number of the current memo: 0, 1 or 2
    float m_fAngMax[7];		// max angle for each joint
    float m_fAngMin[7];		// min angle for each joint
    float m_fDiffToCheck[7];	// write to angMemo if one of the difference of a joint angle exceeds this value
    float m_fSetBackAngle[7];	// angle used to set back by each joint
    JointLimitState m_jointStatus[7]; // result of the limitCheck() is written here
protected:
    int limitCheck ();		// 1: within limit,        -1 : without limit
    void convAng (const ANGLE &angle, float fAngle[]); // ANGLE to float[7]
    void convAng (const float fAngle[], ANGLE &angle); // float[7] to ANGLE
    int diffOver ();		// if one of the joint angle differs to much 1, else 0
    void getCurrAngle();	// get current angle
    void putCurrToMemo();	// put m_fCurAngle to m_fAngMemo
    void copyMemo1toMemo0();	// copy m_fAngMemo[1] to m_fAngMemo[0]
public:
    AngleLimitterImpl();
    void Init (Pa10Controller * controller);
    void Init (Pa10Controller * controller, const float fLimitAngle[7][2]); // limit angles to set, [0]:min, [1]:max
    void DoLimitter (VELMODE vmode, AXIS axes); // vmode is velocity control mode at call time.
				// Non-velocity mode is (VELMODE)0
};
    

#endif // _angle_limitter_h
