/*! @file ArmImpl.h
 *  @brief ArmImpl饹
 */

/* Copyright 1999-2000 Kosei Kitagaki */

/*
modification history
--------------------
01b,11dec02,kk  rename functions
01a,29nov02,kk  first issue
*/

#ifndef _ARMIMPL_H
#define _ARMIMPL_H

/* includes */
#include "SingleArm.h"

#ifdef ARTPALIB
#include <memory.h>
#endif

/*!  @brief  SingleArmμ饹
 *
 */

class ArmImpl : public SingleArm 
{
public:
    ArmImpl();
    virtual ~ArmImpl();
    
//=============== Home and Ready ===============
    void setHome(const AngleX& ang);
    void getHome(AngleX& ang);
    void setReady(const AngleX& ang);
    void getReady(AngleX& ang);
    SA_ERR home(void);
    SA_ERR ready(void);

//=============== Base, Wrist and Tool ===============
    void setBase(const Frame& conf);
    void getBase(Frame& conf);
    void setWrist(const Frame& conf);
    void getWrist(Frame& conf);
    void setTool(const Frame& conf);
    void getTool(Frame& conf);
    
    Frame transformConfArmToWrist(const Frame& conf);
    Frame transformConfArmToTool(const Frame& conf);
    Frame transformConfToolToArm(const Frame& conf);

//=============== Arm control level 0 ===============
  /*! @brief BaseɸϤߤᥫ˥륤󥿥եɸϤ
   *  ߰֡
   */
    virtual SA_ERR getCurrentConfArm(Frame& conf)= 0;
    virtual SA_ERR putReferenceConfArm(const Frame& conf)= 0;
    virtual void getReferenceConfArm(Frame& conf)= 0;
    
    virtual SA_ERR putReferenceVelArm(const DoubleVec &vel)= 0;
    virtual void getReferenceVelArm(DoubleVec &vel)= 0;
  //      virtual SA_ERR getCurrentVelArm(DoubleVec &vel)= 0;
    
    virtual SA_ERR getCurrentAngArm(AngleX& ang7)= 0;
    virtual SA_ERR putReferenceAngArm(const AngleX& ang7)= 0;
    virtual void getReferenceAngArm(AngleX& ang7)= 0;
    
    virtual SA_ERR putReferenceAngVelArm(const AngleX& ang)= 0;
    virtual void getReferenceAngVelArm(AngleX& ang)= 0;
  //      virtual SA_ERR getCurrentAngVelArm(AngleX& ang)= 0;
    
    virtual void setControlCoords(const Frame& controlFrame)= 0;
    virtual void getControlCoords(Frame& controlFrame)= 0;
//=============== Arm control level 1 ===============
  //! @brief WristɸϤθ߰֡
    virtual SA_ERR getCurrentConfWrist(Frame& conf);
    
    virtual SA_ERR getCurrentConf(Frame& conf);
    virtual SA_ERR putReferenceConf(const Frame& conf);
    virtual void getReferenceConf(Frame& conf);
    
    virtual SA_ERR where(Frame& conf);
    virtual SA_ERR move(const Frame& conf);
    virtual void getRef(Frame& conf);

    virtual SA_ERR putReferenceVel(const DoubleVec& conf);
    virtual void getReferenceVel(DoubleVec& conf);
  //	virtual void getCurrentVel(DoubleVec& conf);

    virtual SA_ERR putReferenceAng(const AngleX& ang7);
    virtual void getReferenceAng(AngleX& ang7);
    virtual SA_ERR getCurrentAng(AngleX& ang7);
    
    virtual SA_ERR putReferenceAngVel(const AngleX& ang);
    virtual void getReferenceAngVel(AngleX& ang);
  //	virtual SA_ERR getCurrentAngVel(AngleX& ang);

private:
    Frame m_baseConf;	// base configuration
    Frame m_baseConfInv;	
    Frame m_wristConf;	// wrist configuration
    Frame m_wristConfInv;
    Frame m_toolConf;	// tool configuration
    Frame m_toolConfInv;
    Frame m_wristToolConf;	// wrist * tool configuration
    Frame m_wristToolConfInv;
    
    AngleX *m_homeAng;	// home configuration
    AngleX *m_readyAng;	// ready configuration
    BOOL m_isHomeSet;
    BOOL m_isReadySet;
};	// class ArmImpl

#endif
// EOF
