/*
 * ArtPalib.h
 * Author   : Toshio HORI <toshi@etl.go.jp>
 * Created  : 1999/04/12
 * Modified : 2000/05/16
 * Comment  : PA-10 class.
 *	      - Added methods for velocity.C, redundant.C, and point.C,
 *		which are written by kitagaki@etl.go.jp. (2000/05/16)
 *
 * $Id: ArtPalib.h,v 1.1.1.1 2003/07/14 14:31:49 asahi Exp $
 *
 * Copyright (C) 1999,2000 Toshio HORI.
 */
#ifndef _ARTPALIB_H
#define _ARTPALIB_H

#include <memory.h>
#include <Pa10Controller.h>
#include "IfBoard.h"
#include "pa.h"


//-----------------------------------------------------------------------------

class ArtPalib {
public: //---------------------------------------------------------------------
    // Contructors/Destructor
    ArtPalib(ARM arm = ARM0);	// Default constructor.
    virtual ~ArtPalib(void);		// Destructor.

    // Public functions.

    PA_ERR	    initSystem(void);		// Initialize system.
    PA_ERR	    termSystem(void);		// Terminate system.

    PA_ERR	    openArm(void);		// Open arm.
    PA_ERR	    closeArm(void);	// Close arm.
    PA_ERR	    startArm(void);	// Start arm control.
    PA_ERR	    exitArm(void);	// Finish arm control.
    PA_ERR	    stopArm(SA_WAITSW);		// Stop arm control.
    PA_ERR	    suspendArm(SA_WAITSW);	// Suspend arm control.
    PA_ERR	    restartArm(SA_WAITSW);	// Restart arm control.
    PA_ERR	    execAxis(AXIS, const ANGLE&, SA_WAITSW);	// Exec each axis motion.
    PA_ERR	    moveMatrix(MOVEMODE, const MATRIX&, const ANGLE&, SA_WAITSW);

    PA_ERR	    setPose(const int&);	// Set arm pose: standing/hanging.

    PA_ERR	    resetControl(void);		// Reset control.
    PA_ERR	    errorMessage(PA_ERR, STRING);	// Print error message.

    PA_ERR	    modeAxis(void);		// Each axis control mode.
    PA_ERR	    orderAxis(const ANGLE&);	// Control each axis.
    PA_ERR	    setTimeout(const int&);	// Set timeout in synch. mode.

  //PA_ERR	    getStatus(ARMSTATUS&);		// motion control program version.
    PA_ERR	    getErrorInfo(int&);		// Get error info.
    PA_ERR	    getAngle(ANGLE&);		// Get all axis angle.
    PA_ERR	    getNOA(MATRIX);		// GET (N,O,A,P) matrix.

    PA_ERR	    getMode(int&);		// motion control CPU status.
    PA_ERR	    getRedundant(JOUMODE&);	// redundant axis control mode in RMRC.
    PA_ERR	    getVersion(float&);		// motion control program version.
    PA_ERR	    getTimeout(int&);		// timeout value.


    PA_ERR	    modeVelocity(VELMODE vmode, AXIS axis);

    PA_ERR	    orderVelocity(float speed[]);
    PA_ERR	    modeRedundant(JOUMODE cmode);
    PA_ERR	    orderRedundant(JOUTYPE jt);

  // implement for IfBoard
    PA_ERR inputDIO(DIOSTATUS&);
    PA_ERR outputDIO(DIOSTATUS&);
    PA_ERR setDIO(DIOPORT, DIOCH);
    PA_ERR resetDIO(DIOPORT, DIOCH);


private:
  ARM	    armNo;		// Arm No.
}; /* class ArtPalib */

#endif /* _ARTPALIB_H */

/* EOF */
