/*
 * ArtPalibController.h
 * Author   : Toshio HORI <toshi@etl.go.jp>
 * Created  : 1999/04/12
 * Modified : 2000/05/16
 * Comment  : PA-10 class.
 *	      - Added methods for velocity.C, redundant.C, and point.C,
 *		which are written by kitagaki@etl.go.jp. (2000/05/16)
 *
 * $Id: ArtPalibController.h,v 1.1.1.1 2003/07/14 14:31:49 asahi Exp $
 *
 * Copyright (C) 1999,2000 Toshio HORI.
 */
#ifndef _ARTPALIBCONTROLLER
#define _ARTPALIBCONTROLLER

#include <memory.h>
//#include "pa.h"
#include <Pa10Controller.h>
#include "ArtPalib.h"

//-----------------------------------------------------------------------------

class ArtPalibController : public Pa10Controller, public IfBoard {
public: //---------------------------------------------------------------------

    // Contructors/Destructor (in system.C)
    ArtPalibController(ARM arm = ARM0);	// Default constructor.
    virtual ~ArtPalibController(void);		// Destructor.

    // Public functions.

    PA_ERR	    initSystem(void);		// Initialize system.
    PA_ERR	    termSystem(void);		// Terminate system.

    PA_ERR	    openArm(void);		// Open arm.
    PA_ERR	    closeArm(void);	// Close arm.
    PA_ERR	    startArm(void);	// Start arm control.
    PA_ERR	    exitArm(void);	// Finish arm control.
    PA_ERR	    stopArm(SA_WAITSW);		// Stop arm control.
    PA_ERR	    suspendArm(SA_WAITSW);	// Suspend arm control.
    PA_ERR	    restartArm(SA_WAITSW);	// Restart arm control.
    PA_ERR	    execAxis(AXIS, const ANGLE&, SA_WAITSW);	// Exec each axis motion.
    PA_ERR	    moveMatrix(MOVEMODE, const MATRIX&, const ANGLE&, SA_WAITSW);

    PA_ERR	    setPose(const int&);	// Set arm pose: standing/hanging.

    PA_ERR	    resetControl(void);		// Reset control.
    PA_ERR	    errorMessage(PA_ERR, STRING);	// Print error message.

    PA_ERR	    modeAxis(void);		// Each axis control mode.
    PA_ERR	    orderAxis(const ANGLE&);	// Control each axis.
    PA_ERR	    setTimeout(const int&);	// Set timeout in synch. mode.

  //    PA_ERR	    getStatus(ARMSTATUS&);		// motion control program version.
    PA_ERR	    getErrorInfo(int&);		// Get error info.
    PA_ERR	    getAngle(ANGLE&);		// Get all axis angle.
    PA_ERR	    getNOA(MATRIX);		// GET (N,O,A,P) matrix.

    PA_ERR	    getMode(int&);		// motion control CPU status.
    PA_ERR	    getRedundant(JOUMODE&);	// redundant axis control mode in RMRC.
    PA_ERR	    getVersion(float&);		// motion control program version.
    PA_ERR	    getTimeout(int&);		// timeout value.


    PA_ERR	    modeVelocity(VELMODE vmode, AXIS axis);

    PA_ERR	    orderVelocity(float speed[]);
    PA_ERR	    modeRedundant(JOUMODE cmode);
    PA_ERR	    orderRedundant(JOUTYPE jt);


    BOOL canMoveMatrix(void){return FALSE;};
    void setRestrictAngle(const float[7]){};
    void getRestrictAngle(float[7]){};
    void setNowaitSw(void){};

    //	Servo table accessor
    void doSvOn(const int){};
    void doSvOff(void){};
    void doSvLimitter(const int){};
    void putReferenceConfArm(const Frame&){};
    void putReferenceVelArm(const float[6]){};
    void putReferenceAngArm(const float[7]){};
    void putReferenceAngVelArm(const float[7]){};
    void setControlCoords(const Frame&){};
    void setFGain(const float[6][3]){};
    void setCompParams(const float[6][3]){};
    void getCompParams(float[6][3]){};
    void setComp(const float[6][3]){};
    void setJoyMode(const int){};
    void setJoyVGain(const float[2]){};
    void setJoyFGain(const float[2]){};
    void setLimitAbsVelArm(const float[6]){};
    void setLimitAngArm(const float[7][2]){};
    void setLimitAbsAngVelArm(const float[7]){};
    void setLimitAbsFrc(const float[2]){};
    void setLimitFrc(const float[6][2]){};
    
    int getSvMode(void){return 0;};
    int getSvLimitterMode(void){return 0;};
    void getReferenceConfArm(Frame&){};
    void getReferenceVelArm(float[6]){};
    void getReferenceAngArm(float[7]){};
    void getReferenceAngVelArm(float[7]){};
    void getControlCoords(Frame&){};
    void getFGain(float[6][3]){};
    void getComp(const float[6][3]){};
    int getJoyMode(void){return 0;};
    void getJoyVGain(float[2]){};
    void getJoyFGain(float[2]){};
    void getLimitAbsVelArm(float[2]){};
    void getLimitAngArm(float[7][2]){};
    void getLimitAbsAngVelArm(float[7]){};
    void getLimitAbsFrc(float[2]){};
    void getLimitFrc(float[6][2]){};
    
    int getCounter(void){return 0;};
    int getLoopTime(void){return 0;};
    int getExecTime(void){return 0;};
    int getSvStat(void){return 0;};
    void getCurrentConfArm(Frame&){};
    void getCurrentAngArm(float[7]){};
    void getFrc(float[6]){};
    void getJoyFrc(float[6]){};
    
    BOOL readServo(void){return FALSE;};
    void writeServo(void){};


    // implement for IfBoard
    PA_ERR inputDIO(DIOSTATUS&);
    PA_ERR outputDIO(DIOSTATUS&);
    PA_ERR setDIO(DIOPORT, DIOCH);
    PA_ERR resetDIO(DIOPORT, DIOCH);


protected:

private:
    ArtPalib * palib;
  //    ARM	    armNo;		// Arm No.
  //    int	    clockRate;

}; /* class ArtPalibController */

#endif /* _ARTPALIBCONTROLLER */

/* EOF */
