// myTimer.h - header for world reference time

/*  Copyright 1998 Fumi Saito. */

/*
modification history
--------------------
01b,22apr98,fs   debugged.
01a,20apr98,fs   written.
*/

/*
DESCRIPTION
This header defines the timer interruption class
*/

#ifndef INCmyTimerh
#define INCmyTimerh

#ifdef VXPALIB
#include "taskLib.h"
#include "fppLib.h"
#include "drv/timer/timerDev.h"
#include "semLib.h"
#endif

#define MaxMyTimerN 10

IMPORT void StartMainTimer (double dt);
IMPORT void StopMainTimer ();

class CMyTimer {
private:
    int m_bTimerOn;		// 0 - Timer is OFF, 1 - Timer is ON
    double m_fWTLast;		// last worldTime
    double m_fTStep;		// time step
    int m_iTid;			// task id
    SEM_ID m_semID;		// semaphore id
    int m_iRegIdx;		// index of registration list
    void dispatchSemaphores (); // main semaphore dispatching routine called by a timer interrupt
public:
    CMyTimer(void);
    ~CMyTimer(void);
    STATUS startMainTimer (double dt); // this defines smallest sampling unit
    void stopMainTimer ();
    STATUS connect (VOIDFUNCPTR routine, int arg = 0, int priority = 100);
    STATUS start (double dt);
    void stop ();
    void disconnect ();
    double worldTime();		// utility to get world time
    double dWT();		// utility to get world time step
    double dtNominal();		// utility to get nominal time step
    void clearWT ();		// utility to reset world time to zero
    int mainTimerIsOn ();	// utility to get main timer state
};

class CStopWatch {
    double m_dCurrTime;
    double m_dOffsetTime;
    int m_bMoving;
public:
    void reset (double t);
    void reset () { reset (0.0); }
    void start ();
    void stop ();
    double now ();
    int moving () { return m_bMoving; }
};

#endif /* INCmyTimerh */
