#ifndef __doublevec_h__
#define __doublevec_h__

/*! 
*   @file  DoubleVec.h 
*   @brief DoubleVec class manipulate subroutine library
*
*   [®١®][ϡ⡼]Τ褦ʣ٥ȥ
*
*   Copyright 2000 Kosei Kitagaki 
*/
/*
modification history
--------------------
02c,10nov00,kk  added ftTransform()
02b,08nov00,kk  protected values were changed from SCALAR[6] to Vec * 2
02a,07nov00,kk  renamed from Force to DoubleVec
01b,08nov00,kk  added operator[]
01a,25oct00,kk  written
*/

#include "SubDataType.h"
#include "Frame.h"

/* DoubleVec class */
class DoubleVec {
public:
  DoubleVec();
  DoubleVec(const FLOAT3 f, const FLOAT3 m);
  DoubleVec(const FLOAT6 fv);
  DoubleVec(const Vec&, const Vec&);
  DoubleVec(const SCALAR fx, const SCALAR fy, const SCALAR fz, 
             const SCALAR mx, const SCALAR my, const SCALAR mz);

  //! @brief ǤΥե󥹤֤
  inline SCALAR& operator[] (const int i) { 
    if (i < 0 || 6 <= i)
      cerr << "error ... DoubleVec operator(int) out of range.\n";
    if (i < 3)
    	return f0_[i]; 
    else
    	return f1_[i-3]; 
  }

//    inline Vec& getRefVec1() { return f0_; }
//    inline Vec& getRefVec2() { return f1_; }
//    inline Vec& getRefVel() { return f0_; }
//    inline Vec& getRefRot() { return f1_; }
//    inline Vec& getRefForce() { return f0_; }
//    inline Vec& getRefMoment() { return f1_; }

  //! @brief 
  inline Vec getVec1() const { return f0_; }
  //! @brief 
  inline Vec getVec2() const { return f1_; }
  //! @brief
  inline Vec getVel() const { return f0_; }
  //! @brief
  inline Vec getRot() const { return f1_; }
  //! @brief
  inline Vec getForce() const { return f0_; }
  //! @brief
  inline Vec getMoment() const { return f1_; }

  //! @brief
  inline void setVec1(Vec v) {f0_ = v;}
  //! @brief
  inline void setVec2(Vec v) {f1_ = v;}
  //! @brief
  inline void setVel(Vec v) {f0_ = v;}
  //! @brief
  inline void setRot(Vec v) {f1_ = v;}
  //! @brief
  inline void setForce(Vec v) {f0_ = v;}
  //! @brief
  inline void setMoment(Vec v) {f1_ = v;} 

  void set(const FLOAT6);
  void get(FLOAT6) const;
  void set(const DOUBLE6);
  void get(DOUBLE6) const;

  virtual void clear(void);

  friend DoubleVec operator+(const DoubleVec&, const DoubleVec&);
  friend DoubleVec operator-(const DoubleVec&, const DoubleVec&);
  friend DoubleVec operator*(const SCALAR&, const DoubleVec&);
  friend DoubleVec operator*(const Frame&, const DoubleVec&);
  friend DoubleVec operator/(const DoubleVec&, const SCALAR&);

  virtual DoubleVec& operator+=(DoubleVec&);
  virtual DoubleVec& operator-=(DoubleVec&);
  virtual DoubleVec& operator*=(SCALAR);
  virtual DoubleVec& operator/=(SCALAR);

  virtual DoubleVec rotation(const Frame&);
  virtual DoubleVec ftTransform(const Frame&);
  virtual SCALAR norm1(void);
  virtual SCALAR norm2(void);

  friend ostream& operator<< (ostream&, DoubleVec&);
  //! @brief
  virtual ostream& print() { return cout << *this; };
  virtual ostream& print(ostream&, char*);
  virtual void print (LPTSTR s);

protected:
    Vec f0_, f1_;
};

#endif
