/*!
*  @file  FtSensor.h 
*  @brief FtSensor饹
*
*  Copyright 2002 Kosei Kitagaki 
*/


/*
modification history
--------------------
01b,12dec02,kk  rearrange
01a,29nov02,kk  first issue
*/

#ifndef __ftSensor_h
#define __ftSensor_h

/* includes */
#include "DataType.h"

/*!
*  @class FtSensor
*  @brief FtSensor饹
*/
class FtSensor {
  public:
	//! @brief F/T󥵤
	virtual void init(const int flag) = 0;
	//! @brief F/T󥵤򥭥֥졼󤹤
	virtual void calibration(int fi[6]) = 0;
	//! @brief F/T󥵤֥졼Ѥߤɤ򸡺
	virtual int isCalibrated() = 0;
	//! @brief set F/T sensor base coordinates
	virtual void setFtsBaseCoords(const Frame &fm) = 0;
	//! @brief get F/T sensor base coordinates
	virtual void getFtsBaseCoords(Frame &fm) = 0;
	//! @brief set F/T sensor coordinates for control seen 
	//!        from the F/T sensor base coordinates.
	virtual void setFtsCoords(const Frame &fm) = 0;
	//! @brief get F/T sensor coordinates for control seen 
	//!        from the F/T sensor base coordinates.
	virtual void getFtsCoords(Frame &fm) = 0;

	//! @brief set filter number.
	virtual void setFilter(const int filter) = 0;
	//! @brief get filter number.
	virtual int getFilter(void) = 0;

	//! @brief force/torque at F/T Sensor measurment origin 
	//!        in F/T Sensor measurement coordinates.
	virtual void forceAtFsInFs(DoubleVec& f) = 0;

	//! @brief force at Arm Wrist coordinates (FtsBaseCoords) origin
	//!        in Arm Wrist coordinates (FtsBaseCoords).
	virtual void forceAtAwInAw(DoubleVec& f) = 0;

	//! @brief force at Arm Wrist coordinates (FtsBaseCoords) origin
	//!        in FtsCoords.
	virtual void forceAtAwInFc(DoubleVec& f) = 0;

	//! @brief force at FtsCoords origin
	//!        in FtsCoords.
	virtual void forceAtFcInFc(DoubleVec& f) = 0;

	virtual void transAtFsInFs2AtAwInAw(DoubleVec&, DoubleVec&) = 0;
	virtual void transAtAwInAw2AtFsInFs(DoubleVec&, DoubleVec&) = 0;
	virtual void transAtAwInAw2AtAwInFc(DoubleVec&, DoubleVec&) = 0;
	virtual void transAtAwInAw2AtFcInFc(DoubleVec&, DoubleVec&) = 0;

};

#endif
