/*!
*   @file  FtSensorImpl.h
*   @brief FtSensorImpl饹
*
* Copyright 2002 Kosei Kitagaki 
*/

/*
modification history
--------------------
01b,12dec02,kk  rearrange
01a,29nov02,kk  first issue
*/

#ifndef __ftSensorImpl_h
#define __ftSensorImpl_h

/* includes */
#include "DataType.h"
#include "FtSensor.h"

/*!
*   @class  FtSensorImpl 
*   @brief  FtSensorImpl饹
*/
class FtSensorImpl : public FtSensor {
  public:
	FtSensorImpl();
	virtual void init(const int flag) = 0;
	virtual void calibration(int fi[6]) = 0;
	virtual int isCalibrated();

	virtual void setFtsBaseCoords(const Frame &fm);
	virtual void getFtsBaseCoords(Frame &fm);
	virtual void setFtsCoords(const Frame &fm);
	virtual void getFtsCoords(Frame &fm);
//	virtual void setFtsEndCoords(const Frame &fm);
//	virtual void getFtsEndCoords(Frame &fm);

	virtual void setFilter(const int filter);
	virtual int getFilter(void);

	//! @brief force/torque at F/T Sensor measurment origin 
	//!        in F/T Sensor measurement coordinates.
	virtual void forceAtFsInFs(DoubleVec& f) = 0;

	//! @brief force at Arm Wrist coordinates (FtsBaseCoords) origin
	//!        in Arm Wrist coordinates (FtsBaseCoords).
	virtual void forceAtAwInAw(DoubleVec& f) = 0;

	//! @brief force at Arm Wrist coordinates (FtsBaseCoords) origin
	//!        in FtsCoords.
	virtual void forceAtAwInFc(DoubleVec& f) = 0;

	//! @brief force at FtsCoords origin
	//!        in FtsCoords.
	virtual void forceAtFcInFc(DoubleVec& f) = 0;

	virtual void transAtFsInFs2AtAwInAw(DoubleVec&, DoubleVec&) = 0;
	virtual void transAtAwInAw2AtFsInFs(DoubleVec&, DoubleVec&) = 0;
	virtual void transAtAwInAw2AtAwInFc(DoubleVec&, DoubleVec&) = 0;
	virtual void transAtAwInAw2AtFcInFc(DoubleVec&, DoubleVec&) = 0;

  protected:
	int calibrated;
	Frame myFtsBaseCoords;		// Ftsensor base coordinates
	Frame myFtsBaseCoordsInv;	//  inverse
	Frame myFtsCoords;		// Ftsensor coordinates
	Frame myFtsCoordsInv;		//  inverse
//	Frame myFtsEndCoords;		// Ftsensor end-effecor side flange
//	Frame myFtsEndCoordsInv;	//  inverse
	int myFilterNo;			// Ftsensor filter number	
};

#endif
