#ifndef __matrix3x3_h__
#define __matrix3x3_h__
/*!
*  @file  Matrix3x3.h
*  @brief 3x3󥯥饹
*
*  Copyright 2002 Kosei Kitagaki 
*/
/*	
modification history
--------------------
01a,31oct02,kk  first issue
*/

#include "SubDataType.h"
#include "Vec.h"

/*!
*  @class  Matrix3x3
*  @brief 3x3󥯥饹
*
*/
class Matrix3x3 {
public:
  Matrix3x3();
  Matrix3x3(const FLOAT3x3 m);
  Matrix3x3(const DOUBLE3x3 m);

  inline void setElement(int i, int j, SCALAR element) 
	{ matrix[i][j] = element; };
  inline SCALAR getElement(int i, int j) const
	{ return matrix[i][j]; };
  void get(FLOAT3x3 array) const;
  void set(const FLOAT3x3 array);
  void get(DOUBLE3x3 array) const;
  void set(const DOUBLE3x3 array);
  //SCALAR3x3& getRef(void) { return matrix; };

  void clear(void);
  void unit(void);

  friend Matrix3x3 operator+ (const Matrix3x3&, const Matrix3x3&);
  friend Matrix3x3 operator- (const Matrix3x3&, const Matrix3x3&);
  friend Matrix3x3 operator* (const Matrix3x3&, const Matrix3x3&);
  friend Matrix3x3 operator* (const SCALAR&, const Matrix3x3&);
  friend Matrix3x3 operator* (const Matrix3x3&, const SCALAR&);
  friend Matrix3x3 operator/ (const Matrix3x3&, const SCALAR&);
  friend Vec operator* (const Matrix3x3&, const Vec&);

  virtual Matrix3x3& operator+= (Matrix3x3&);
  virtual Matrix3x3& operator-= (Matrix3x3&);
  virtual Matrix3x3& operator*= (SCALAR&);
  virtual Matrix3x3& operator/= (SCALAR&);

  virtual Matrix3x3 transpose(void);

  friend ostream& operator<< (ostream&, Matrix3x3&);
  virtual ostream& print(void) { return cout << *this; };
  virtual void print (LPTSTR s);

protected:
  SCALAR3x3 matrix;
};

/*!
*  @class  RotationMatrix
*  @brief ž󥯥饹
*
*/
class RotationMatrix : public Matrix3x3
{
public:
  RotationMatrix():Matrix3x3(){};
  RotationMatrix(Matrix3x3 m):Matrix3x3(m) {};
  RotationMatrix(const FLOAT3x3 m):Matrix3x3(m) {};
  RotationMatrix(const DOUBLE3x3 m):Matrix3x3(m) {};
  RotationMatrix inverse(void);
  RotationMatrix& operator= (Matrix3x3 m);
};

#endif
