//	Structure definitions for Nitta force sensor
//
//	3.2,05nov02,kk  class FtSensor -> NittaImpl
//	3.1,10apr00,kk	add sensors 3 and 4

#define	DEBUG

//==============================================================
//	Force sensor id number
//==============================================================
enum { NITTA0, NITTA1, NITTA2, NITTA3  };

//==============================================================
//	For Nitta force sensor
//==============================================================
//	Base addresses	
//#define NITTA_FTSENSOR_BASE0	0x7e608000
//#define NITTA_FTSENSOR_BASE1	0x7e618000
#define NITTA_FTSENSOR_BASE0	0x7f608000
#define NITTA_FTSENSOR_BASE1	0x7f618000
#define NITTA_FTSENSOR_BASE2	0x7f628000
#define NITTA_FTSENSOR_BASE3	0x7f638000
#define VME_NITTA_FTSENSOR_BASE0	0xFF608000
#define VME_NITTA_FTSENSOR_BASE1	0xFF618000
#define VME_NITTA_FTSENSOR_BASE2	0xFF628000
#define VME_NITTA_FTSENSOR_BASE3	0xFF638000

typedef unsigned char   u_char;
typedef unsigned short  u_short;
typedef unsigned int    u_int;
typedef unsigned long   u_long;

#define CalibrTimes	100

typedef enum	
{
	lbs_in_lbs_mils,
	N_dNm_mmx10,
	kgfx10_kgFcm_mmx10,
	klbs_kin_lbs_mils,
	reserved_units_4,
	reserved_units_5,
	reserved_units_6,
	reserved_units_7
} FORCE_UNITS;

typedef	enum
{
	end_x_form,
	tx,
	ty,
	tz,
	rx,
	ry,
	rz,
	neg
} LINK_TYPES;


//	FORCE_UNITS	
//
//#define	lbs_in_lbs_mils		0
//#define	N_dNm_mmx10		1
//#define	kgfx10_kgFcm_mmx10	2
//#define	klbs_kin_lbs_mils	3
//#define	reserved_units_4	4
//#define	reserved_units_5	5
//#define	reserved_units_6	6
//#define	reserved_units_7	7

//	LINK_TYPES	
//
//#define	END_X_FORM	0
//#define	TX		1
//#define	TY		2
//#define	TZ		3
//#define	RX		4
//#define	RY		5
//#define	RZ		6
//#define	NEG		7

//	VECT_BITS
#define	VECT_BITS_FX	0x0001
#define	VECT_BITS_FY	0x0002
#define	VECT_BITS_FZ	0x0004
#define	VECT_BITS_MX	0x0008
#define	VECT_BITS_MY	0x0010
#define	VECT_BITS_MZ	0x0020
#define	VECT_BITS_CHANGEV1	0x0040
#define	VECT_BITS_CHANGEV2	0x0080

//	WARNING_BITS	
#define	WARNING_BITS_FX_NEAR_SAT	0x0001
#define	WARNING_BITS_FY_NEAR_SAT	0x0002
#define	WARNING_BITS_FZ_NEAR_SAT	0x0004
#define	WARNING_BITS_MX_NEAR_SAT	0x0008
#define	WARNING_BITS_MY_NEAR_SAT	0x0010
#define	WARNING_BITS_MZ_NEAR_SAT	0x0020

//	ERROR_BITS	
#define	ERROR_BITS_FX_SAT	0x0001
#define	ERROR_BITS_FY_SAT	0x0002
#define	ERROR_BITS_FZ_SAT	0x0004
#define	ERROR_BITS_MX_SAT	0x0008
#define	ERROR_BITS_MY_SAT	0x0010
#define	ERROR_BITS_MZ_SAT	0x0020
#define	ERROR_BITS_MEMORY_ERROR	0x0400
#define	ERROR_BITS_SENSOR_CHARGE 0x0800
#define	ERROR_BITS_SYSTEM_BUSY	0x1000
#define	ERROR_BITS_CAL_CRC_BAD	0x2000
#define	ERROR_BITS_WATCH_DOG1	0x4000
#define	ERROR_BITS_WATCH_DOG	0x8000

typedef struct
{
	u_short	raw_time;
	short	raw_data;
	short	reserved[2];
} RAW_CHANNEL;
typedef struct
{
	short	fx;
	short	fy;
	short	fz;
	short	mx;
	short	my;
	short	mz;
	short	v1;
	short	v2;
} FORCE_ARRAY;
typedef struct
{
	short	fx;
	short	fy;
	short	fz;
	short	mx;
	short	my;
	short	mz;
} SIX_AXIS_ARRAY;
typedef struct
{
	short	data_address;
	short	threshold;
	short	bit_pattern;
} THRESH_STRUCT;
typedef struct
{
	short	latch_bits;
	short	number_of_ge_thresholds;
	short	number_of_le_thresholds;
	THRESH_STRUCT	thresholds[4];
	short	reserved;
} LE_STRUCT;
typedef struct
{
	short	link_type;
	short	link_amount;
} LINKS;
typedef struct
{
	LINKS	link[8];
} TRANSFORM;

typedef struct
{
	RAW_CHANNEL	raw_channels[16];	/* offset 0x0000 */
	short		copyright[0x0018];	/* offset 0x0040 */
	short		reserved1[0x0008];	/* offset 0x0058 */
	SIX_AXIS_ARRAY	shunts;		/* offset 0x0060 */
	short		reserved2[2];	/* offset 0x0066 */
	SIX_AXIS_ARRAY	default_fs;	/* offset 0x0068 */
	short		reserved3;	/* offset 0x006e */
	short		load_envelope_num;	/* offset 0x006f */
	SIX_AXIS_ARRAY	min_full_scale;	/* offset 0x0070 */
	short		reserved4;	/* offset 0x0076 */
	short		transform_num;	/* offset 0x0077 */
	SIX_AXIS_ARRAY	max_full_scale;	/* offset 0x0078 */
	short		reserved5;	/* offset 0x007e */
	short		peak_address;	/* offset 0x007f */
	FORCE_ARRAY	full_scale;	/* offset 0x0080 */
	SIX_AXIS_ARRAY	offsets;	/* offset 0x0088 */
	short		offset_num;	/* offset 0x008e */
	u_short		vect_axes;	/* offset 0x008f */
	FORCE_ARRAY	filter[7];	/* offset 0x0090 */
	FORCE_ARRAY	rate_data;	/* offset 0x00c8 */
	FORCE_ARRAY	minimum_data;	/* offset 0x00d0 */
	FORCE_ARRAY	maximum_data;	/* offset 0x00d8 */
	short		near_sat_value;	/* offset 0x00e0 */
	short		sat_value;	/* offset 0x00e1 */
	short		rate_address;	/* offset 0x00e2 */
	u_short		rate_divisor;	/* offset 0x00e3 */
	u_short		rate_count;	/* offset 0x00e4 */
	short		command_word2;	/* offset 0x00e5 */
	short		command_word1;	/* offset 0x00e6 */
	short		command_word0;	/* offset 0x00e7 */
	u_short		count[6];	/* offset 0x00e8 */
	u_short		error_count;	/* offset 0x00ee */
	u_short		count_x;	/* offset 0x00ef */
	u_short		warnings;	/* offset 0x00f0 */
	u_short		errors;		/* offset 0x00f1 */
	short		threshold_bits;	/* offset 0x00f2 */
	short		last_crc;	/* offset 0x00f3 */
	short		eeprom_ver_no;	/* offset 0x00f4 */
	short		software_ver_no;/* offset 0x00f5 */
	short		software_day;	/* offset 0x00f6 */
	short		software_year;	/* offset 0x00f7 */
	u_short		serial_no;	/* offset 0x00f8 */
	u_short		model_no;	/* offset 0x00f9 */
	short		cal_day;	/* offset 0x00fa */
	short		cal_year;	/* offset 0x00fb */
	u_short		units;		/* offset 0x00fc */
	short		bit;		/* offset 0x00fd */
	short		channels;	/* offset 0x00fe */
	short		thickness;	/* offset 0x00ff */
	LE_STRUCT	load_envelopes[0x10];	/* offset 0x0100 */
	TRANSFORM	transforms[0x10];	/* offset 0x0200 */
} FORCE_SENSOR_DATA;

//	Definitions of functions	
//**************************************************************
//	level 0 io functions
//**************************************************************
//	Provided command codes for Nitta force sensor	
#define	C_FTN_READ_DATA		(short)0x0100
#define	C_FTN_WRITE_DATA	(short)0x0200
#define	C_FTN_BIT_SET		(short)0x0300
#define	C_FTN_BIT_RESET		(short)0x0400
#define	C_FTN_SET_TRANSFORM	(short)0x0500
#define	C_FTN_SET_OFFSET_NUM	(short)0x0600
#define	C_FTN_SET_OFFSET	(short)0x0700
#define	C_FTN_RESET_OFFSET	(short)0x0800
#define	C_FTN_SET_VECTOR	(short)0x0900
#define	C_FTN_SET_FULL_SCALE	(short)0x0A00
#define	C_FTN_RESET_PEAK	(short)0x0B00
#define	C_FTN_READ_PEAK		(short)0x0C00

class NittaFtsController
{
private:
	int fts_no;
	FORCE_SENSOR_DATA *p_frc;
	float resolutions[6];
	short zero[6];
	char *base_addr;

    void taskDelay(int);

//**************************************************************
//	for ArtPalib Art
//**************************************************************
	int fdq;
	int error;
	//	FILE *fd1;
	unsigned int j;
	unsigned int addr1;
	unsigned int ifs_data;
	//	short data;
	//	int i = 0;
	//	int k = 0;
	unsigned long st, len, poff;
	unsigned long ifs_base_addr;
	short *mmaped;
	int err;
	int fdMem;
	unsigned long MappedAddress; 

public:
	NittaFtsController(int no);
	virtual ~NittaFtsController();
	short read_data(short addr);
	short write_data(short addr, short data);
	short bit_set(short addr, short bit_map);
	short bit_reset(short addr, short bit_map);
	int set_transform(TRANSFORM *transforms, int slot_num);
	int set_offset_num(int offset_num);
	SIX_AXIS_ARRAY  *set_offset(SIX_AXIS_ARRAY *offsets);
	SIX_AXIS_ARRAY  *reset_offset(void);
	short set_vector(int bitmap);
	void set_full_scale(FORCE_ARRAY *full_scale);
	void reset_peak(short peak_addr, FORCE_ARRAY *min, FORCE_ARRAY *max);
	void read_peak(short peak_addr, FORCE_ARRAY *min, FORCE_ARRAY *max);

//**************************************************************
//	level 1 io functions
//**************************************************************
	short  *get_shunts(short frc[6]);
	short get_load_envelope_num(void);
	short set_load_envelope_num(short num);
	short get_transform_num(void);
	short set_transform_num(short num);
	short get_peak_address(void);
	short set_peak_address(short num);
	short  *get_def_full_scale(short frc[6]);
	short  *get_min_full_scale(short frc[6]);
	short  *get_max_full_scale(short frc[6]);
	short  *get_full_scale(short frc[6]);
	short  *get_full_scale_v(short frc[2]);
	short  *set_full_scale1(short frc[6]);
	short  *set_full_scale1_v(short frc[2]);
	short  *get_offsets(short frc[6]);
	short  *set_offsets(short frc[6]);
	short get_offset_num(void);
	short set_offset_num1(short num);
	u_short get_vect_axes(void);
	u_short set_vect_axes(u_short num);
	short  *get_force(int n, short frc[8]);
	short  *get_force0(short frc[8]);
	short  *get_force1(short frc[8]);
	short  *get_force2(short frc[8]);
	short  *get_force3(short frc[8]);
	short  *get_force4(short frc[8]);
	short  *get_force5(short frc[8]);
	short  *get_force6(short frc[8]);
	short  *get_rate_data(short frc[8]);
	short  *get_minimum_data(short frc[8]);
	short  *get_maximum_data(short frc[8]);
	short get_near_sat_value(void);
	short get_sat_value(void);	
	short get_rate_address(void);
	u_short get_rate_divisor(void);
	u_short get_rate_count(void);	
	short set_near_sat_value(short val);
	short set_sat_value(short val);
	short set_rate_address(short val);
	u_short set_rate_divisor(u_short val);
	u_short set_rate_count(u_short val);
	int get_count(int);
	u_short get_count1(void);
	u_short get_count2(void);
	u_short get_count3(void);
	u_short get_count4(void);
	u_short get_count5(void);
	u_short get_count6(void);
	u_short get_error_count(void);
	u_short get_count_x(void);	
	u_short get_warnings(void);
	u_short get_errors(void);	
	short get_threshold_bits(void);	
	short get_last_crc(void);	
	short get_rom_no(void);
	short get_software_no(void);		
	short get_software_day(void);
	short get_software_year(void);	
	short get_serial_no(void);	
	short get_model_no(void);		
	short get_cal_day(void);
	short get_cal_year(void);	
	short get_units(void);	
	short get_bit(void);	
	short get_channels(void);		
	short get_thickness(void);
	char  *get_copyright(char *str);
	LE_STRUCT  *get_load_envelope(int num);
	LE_STRUCT  *set_load_envelope(int num, LE_STRUCT *env);
	TRANSFORM  *get_transforms(int num);
	TRANSFORM  *set_transforms(int num, TRANSFORM *trn);

//**************************************************************
//	level 2 io functions
//**************************************************************
	char *getUnits(float units[6], float *unit2);
	float *getForce(int n, float frc[6]);
	float *getForce0(float frc[6]);
	float *getForce1(float frc[6]);
	float *getForce2(float frc[6]);
	float *getForce3(float frc[6]);
	float *getForce4(float frc[6]);
	float *getForce5(float frc[6]);
	float *getForce6(float frc[6]);
	void setFullScale(float scale[6]);
	void getFullScale(float scale[6]);
	void setDefFullScale(void);
	float *getResolution(float resolutions[6]);
	void resetOffset(int *);
	int initResolution(int com);
	void calibration(int *);

//**************************************************************
//	for ArtPalib Art
//**************************************************************
#if (defined ART) || (defined ARTPALIB)
	void IFS_Write( unsigned int , int );
	short IFS_Read( unsigned int );
#endif
};

/*
 $Log: NittaFtsController.h,v $
 Revision 1.4  2003/09/10 06:46:50  yoon
 *** empty log message ***

 Revision 1.3  2003/09/04 10:07:57  kitagaki
 *** empty log message ***

 Revision 1.2  2003/09/03 02:48:13  yoon
 *** empty log message ***

 Revision 1.2  2001/09/13 06:13:36  root
 Added geting board address.

 Revision 1.1  2001/08/30 01:10:37  root
 Initial revision

*/

#if (defined ART) || (defined ARTPALIB)

#ifndef _SCANINFO_H
#define _SCANINFO_H
//#include <linux/module.h>

#define JR3_READ		(200)
#define JR3_WRITE		(201)
#define JR3_SET_ADDR		(202)
#define JR3_SELECT1		(203)
#define JR3_SELECT2		(204)
#define JR3_SELECT3		(205)
#define JR3_SELECT4		(206)

#define JR3_GET_BADDR1		(207)
#define JR3_GET_BADDR2		(208)
#define JR3_GET_BADDR3		(209)
#define JR3_GET_BADDR4		(210)

#endif /* _SCANINFO_H */

#endif
