/*!
 *  @file  Pa10.h 
 *  @brief PA10饹
 *
 *  @date 2003.3.14
 *  Copyright 2003 Kosei Kitagaki 
 */

#ifndef _PA10_H
#define _PA10_H

#if (defined VXPALIB) || (defined ARTPALIB)
#ifndef PALIB
#define PALIB
#endif
#endif

/* includes */
#include "SingleArm.h"
#include "ArmImpl.h"
#include "Pa10Controller.h"
#ifdef PALIB
#include "PalibController.h"
#else
#include "ArtController.h"
#endif
#include "MiscDevice.h"

/*!
 *  @class  Pa10 
 *  @brief PA10饹ѤɸϤϡŪˡ
 *  ١ɸϤߤᥫ˥륤󥿥եɸϡ
 */
class Pa10 : public ArmImpl {
public:
    Pa10(ARM no = ARM0);
    ~Pa10() {};

//--------------- Utilites ---------------
    void setController(Pa10Controller*);
    Pa10Controller* getController(void);
#ifdef ARTPALIB
    void setPalibController(PalibController*);
    PalibController* getPalibController(void);
#endif
    void setLamp(Lamp*);
    Lamp* getLamp(void);
        
//=============== Basic control ==============
    void init(int pose);
    SA_ERR wake(void);
    SA_ERR stop(void);
    SA_ERR sleep(void);
    SA_ERR term(void);		// Terminate system.
    SA_ERR getStat(int&);
    SA_ERR getError(void);

    void setNowaitSw(void);

    SA_ERR suspend(SA_WAITSW);   // Suspend arm control.
    void setArmNumber(ARM arm);
    ARM getArmNumber(void);
    void setRestrictAngle(const AngleX&);
    void getRestrictAngle(AngleX&);
    SA_ERR setRedundantMode(JOUMODE mode);
    JOUMODE getRedundantMode(void);
    SA_ERR setRedundantOrder(JOUTYPE jtype);
    JOUTYPE getRedundantOrder(void);
    SA_ERR resetControl(void);		// Reset control.
    SA_ERR getVersion(float&);		// motion control program version.

//=============== Arm control level 0 ===============
    SA_ERR getCurrentConfArm(Frame& conf);
    SA_ERR putReferenceConfArm(const Frame& conf);
    void getReferenceConfArm(Frame& conf);

    SA_ERR putReferenceVelArm(const DoubleVec &vel);
    void getReferenceVelArm(DoubleVec &vel);
//    SA_ERR getCurrentVelArm(DoubleVec &vel);

    SA_ERR getCurrentAngArm(AngleX& angX);
    SA_ERR putReferenceAngArm(const AngleX& angX);
    void getReferenceAngArm(AngleX& angX);

    SA_ERR putReferenceAngVelArm(const AngleX& angX);
    void getReferenceAngVelArm(AngleX& angX);
//    SA_ERR getCurrentAngVelArm(AngleX& ang);
//    SA_ERR getCurrentConfWrist(Frame& conf);

    void setControlCoords(const Frame& controlFrame);
    void getControlCoords(Frame& controlFrame);
    
//=============== Force control ===============
    void doForceControl(const DoubleVec&);

protected:
    SA_ERR putReferenceConfArm(const Frame& conf, SA_WAITSW sw);

private:
    ARM armNo_;		// Arm Number
//    Angle7 restrictAng_;    // restrict angles for redundant mode
    JOUMODE rMode_;     // Redundant control mode (used in wake())
    JOUTYPE rType_;     // Redundant control type  
//    Frame referenceConf_;   // reference of Configuration 
//    Angle7 referenceAng7_;  // reference of Angles
  
    Pa10Controller *controller;
  //    PalibController *palibController;
    Lamp * lamp;
};  // class Pa10

#endif
// EOF
