/*
 * Palib.h
 * Author   : Toshio HORI <toshi@etl.go.jp>
 * Created  : 1999/04/12
 * Modified : 2000/05/16
 * Comment  : PA-10 class.
 *	      - Added methods for velocity.C, redundant.C, and point.C,
 *		which are written by kitagaki@etl.go.jp. (2000/05/16)
 *
 * $Id: Palib.h,v 1.8 2003/11/14 08:32:34 kitagaki Exp $
 *
 * Copyright (C) 1999,2000 Toshio HORI.
 */
#ifndef _PALIB_H
#define _PALIB_H

typedef long PA_ERR;
#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif

#ifdef ARTPALIB
#include <stdlib.h>
#include <memory.h>
//#include "pa.h"
#include "ArtPalib/pa.h"
#include "ArtPalib/paerr.h"
typedef PA_ERR	    (*WFP)(int); // WFPƤ롪
#endif // ARTPALIB

#ifdef ART
#include <memory.h>
//#include "pa.h"
#include "ArtPalib/pa.h"
#include "ArtPalib/paerr.h"
//typedef unsigned long ARM;
typedef PA_ERR	    (*WFP)(int); // WFPƤ롪
#endif // ART

#ifdef ARTWRAP
#include <memory.h>
//#include "pa.h"
#include "ArtPalib/pa.h"
#include "ArtPalib/paerr.h"
//typedef unsigned long ARM;
typedef PA_ERR	    (*WFP)(int); // WFPƤ롪
#endif // ARTWRAP

#ifdef VXPALIB
#include <stdio.h>
#include <stdlib.h>

/* Include files for VxWorks/Tornado */
#include <taskLib.h>
#include <kernelLib.h>
#include <in.h>
#include <vme.h>
#include <sysLib.h>
#include <time.h>

#include "VxPalib/patype.h"
#include "VxPalib/paJoint.h"
#include "VxPalib/pa_error.h"
#include "VxPalib/pacmd.h"
#include "VxPalib/pammc++.h"

/*
 *	JUMPåоDI
 */
typedef unsigned long	DIOKIND;
#define	DIO_INTERNAL	(DIOKIND)0x00000000
#define	DIO_EXTERNAL	(DIOKIND)0x00000001

//long pa_inp_dio(ARM armNo, DIOKIND kind, DIOSTATUSP stat);
//long pa_oup_dio(ARM armNo, DIOKIND kind, DIOSTATUSP stat);

#endif // VXPALIB

//#include <Pa10Controller.h>
//#include <IfBoard.h>

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

class Palib {
public: //---------------------------------------------------------------------

    // Contructors/Destructor (in system.cpp)
    Palib(ARM arm = ARM0);	// Default constructor.
    virtual ~Palib() {};	// Destructor.

    // Public functions.

    // in system.cpp
    static PA_ERR   initSystem(void);		// Initialize system.
    PA_ERR	    termSystem(void);		// Terminate system.

    // in motion.cpp
    PA_ERR	    openArm(void);		// Open arm.
    PA_ERR	    closeArm(void);	// Close arm.
    PA_ERR	    startArm(void);	// Start arm control.
    PA_ERR	    exitArm(void);	// Finish arm control.
    PA_ERR	    stopArm(WFP);		// Stop arm control.
    PA_ERR	    suspendArm(WFP);	// Suspend arm control.
    PA_ERR	    restartArm(WFP);	// Restart arm control.
    PA_ERR	    execAxis(AXIS, const ANGLE&, WFP);	// Exec each axis motion.
    PA_ERR	    moveMatrix(MOVEMODE, const MATRIX&, const ANGLE&, WFP);

    // in direct.C
    PA_ERR	    setPose(const int&);	// Set arm pose: standing/hanging.

    // in misc.cpp
    PA_ERR	    resetControl(void);		// Reset control.
    PA_ERR	    errorMessage(PA_ERR, unsigned char *);	// Print error message.

    // in support.cpp
    PA_ERR	    modeAxis(void);		// Each axis control mode.
    PA_ERR	    orderAxis(const ANGLE&);	// Control each axis.
    PA_ERR	    setTimeout(const int&);	// Set timeout in synch. mode.

    // in status.cpp
    PA_ERR          getStatus(ARMSTATUS&);	// Get arm status
    PA_ERR          getCounter(int&);
    PA_ERR	    getErrorInfo(int&);		// Get error info.
    PA_ERR	    getAngle(ANGLE&);		// Get all axis angle.
    PA_ERR	    getNOA(MATRIX);		// GET (N,O,A,P) matrix.

    // in extstat.cpp / pa_egt.c
    PA_ERR	    getMode(int&);		// motion control CPU status.
#ifdef ARTPALIB
    PA_ERR          getCom(int&);               // communication mode
#endif
    PA_ERR	    getRedundant(JOUMODE&);	// redundant axis control mode in RMRC.
    PA_ERR	    getVersion(float&);		// motion control program version.
    PA_ERR	    getTimeout(int&);		// timeout value.

    // in velocity.cpp
    PA_ERR	    modeVelocity(VELMODE vmode, AXIS axis);

    // in redundant.cpp
    PA_ERR	    orderVelocity(float speed[]);
    PA_ERR	    modeRedundant(JOUMODE cmode);
    PA_ERR	    orderRedundant(JOUTYPE jt);

    // implement IfBoard in dio.cpp
    PA_ERR inputDIO(DIOSTATUS& ds);
    PA_ERR outputDIO(DIOSTATUS& ds);
    PA_ERR setDIO(DIOPORT port, DIOCH channel);
    PA_ERR resetDIO(DIOPORT port, DIOCH channel);

    ARM getArmNumber() { return armNo; }
private:
    ARM	    armNo;		// Arm No.
#ifdef VXPALIB
    int	    clockRate;
    PA_ERR	    curErr;		// current error;
    COMMptr commP;		// Pointer to the communication memory.
    char*   cueP;		// Pointer for command cue.
    int	    status;
    int	    statusArr[MAX_ARM];
    int	    reg3;
    int	    reg3Arr[MAX_ARM];

    // Private functions. (in private.cpp)
    PA_ERR	    request(int);	// Request command.

    void    copyInteger(int&, const int&);
    int	    getInteger(const int&);
    float   getFloat(const float&);
    char    getChar(const char&);
    void    putInteger(int&, const int&);
    void    putFloat(float&, const float&);
    void    putChar(char&, const char&);
    void    memCopy(void*, void*, size_t);

    PA_ERR	    getError(void);

    PA_ERR	    setPointers(void);	// Set commP and cueP pointers.
    int	    isFinish(void);	// Check event finish.
#endif // VXPALIB
    static int m_initP;
    int m_openP;
    int m_startP;
}; 

#endif /* _PALIB_H */

/* EOF */
