#ifndef __INCSharedTable_h
#define __INCSharedTable_h

//#include "DataType.h"

/* SharedTable.h - shared buffer table controller */
/* Copyright 2003 Kosei Kitagaki */
/*
modification history
--------------------
02a,11mar03,kk	make Creator which manage multiple tables of same type
01b,10mar03,kk  style refined
01a,27feb03,kk  written
*/

#if (__GNUC__ == 2) && (__GNUC_MINOR__ == 7)
#define GCC_2_7
#endif

#ifndef NULL
#define NULL (0)
#endif

#define MAX_TABLE 10

template<typename AType> 
  class SharedTable
  {
  public:
	void writeSharedTable(AType* theTable);
	BOOL readSharedTable(AType* theTable);
	int statSharedTable();

  protected:
	static SharedTable* instanceSharedTable();
	SharedTable();

  private:
#ifdef GCC_2_7
	static SharedTable<AType>* mySharedTable = NULL;
#else
	static SharedTable<AType>* mySharedTable;
#endif
	int dataAvailable;
	AType tbl[2];
  };

#ifndef GCC_2_7
template<typename AType> 
  SharedTable<AType>*
  SharedTable<AType>::mySharedTable = NULL;
#endif

#if 0
template<typename AType> 
  SharedTable<AType>*
  SharedTable<AType>::mySharedTable;
#endif

template<typename AType>
  SharedTable<AType>*
  SharedTable<AType>::instanceSharedTable() 
  {
	mySharedTable = new SharedTable();
	return mySharedTable;
  }

template<typename AType> 
  SharedTable<AType>::SharedTable() 
  {
	dataAvailable = -1;
  }

template<typename AType> 
  void
  SharedTable<AType>::writeSharedTable(AType* theTable) 
  {
	if (dataAvailable == 0) 
	{
		tbl[1] = *theTable;
		dataAvailable = 1;
	} 
	else 
	{
		tbl[0] = *theTable;
		dataAvailable = 0;
	}
  }

template<typename AType> 
  BOOL
  SharedTable<AType>::readSharedTable(AType* theTable) 
  {
	if (dataAvailable == 0) 
	{
		*theTable = tbl[0];
	} 
	else if (dataAvailable == 1) 
	{
		*theTable = tbl[1];
	} 
	else 
	{
		return FALSE;
	}
	return TRUE;
  }

template<typename AType> 
  int
  SharedTable<AType>::statSharedTable() 
  {
	return dataAvailable;
  }

//	class SharedTableCreator

template<typename AType> 
  class SharedTableCreator : public SharedTable<AType>
  {
  public:
	static SharedTable<AType>* createSharedTable(int n);

  protected:
	SharedTableCreator();

  private:
#ifdef GCC_2_7
	static SharedTableCreator<AType>* mySharedTableCreator = NULL;
#else
	static SharedTableCreator<AType>* mySharedTableCreator;
#endif
	static SharedTable<AType>* mySharedTables[MAX_TABLE];
  };

#ifndef GCC_2_7
template<typename AType> 
  SharedTableCreator<AType>*
  SharedTableCreator<AType>::mySharedTableCreator = NULL;

template<typename AType> 
  SharedTable<AType>*
  SharedTableCreator<AType>::mySharedTables[MAX_TABLE];
#endif

template<typename AType> 
  SharedTableCreator<AType>::SharedTableCreator()
  {
	int i;
	for (i = 0; i < MAX_TABLE; i++)
		mySharedTables[i] = NULL;
  }

template<typename AType> 
  SharedTable<AType>*
  SharedTableCreator<AType>::createSharedTable(int n)
  {
	if (mySharedTableCreator == NULL) 
	{
		mySharedTableCreator = new SharedTableCreator();
	}
	if ((n < 0) && (MAX_TABLE <= n))
	{ 
		return NULL;
	}
	if (mySharedTables[n] == NULL)
	{
		mySharedTables[n] = SharedTable<AType>::instanceSharedTable();
//		mySharedTables[n] = new SharedTable<AType>::SharedTable();
	}
	return mySharedTables[n];
  }

#endif // __INCSharedTable_h
// end of SharedTable.h
