#ifndef __INCSharedTable_h
#define __INCSharedTable_h

/* SharedTable.h - shared buffer table controller */
/* Copyright 2003 Kosei Kitagaki */
/*
modification history
--------------------
01b,10mar03,kk  style refined
01a,27feb03,kk  written
*/

#if (__GNUC__ == 2) && (__GNUC_MINOR__ == 7)
#define GCC_2_7
#endif

#ifndef BOOL
typedef int BOOL;
#endif

#ifndef TRUE
#define TRUE (1)
#define FALSE (0)
#endif

#ifndef NULL
#define NULL (0)
#endif

template<typename AType> 
  class SharedTable
  {
  public:
	static SharedTable<AType> *instanceSharedTable();
	void writeSharedTable(AType *theTable);
	BOOL readSharedTable(AType *theTable);
	int statSharedTable();

  protected:
	SharedTable();

  private:
#ifdef GCC_2_7
	static SharedTable<AType> *mySharedTable = NULL;
#else
	static SharedTable<AType> *mySharedTable;
#endif
	int dataAvailable;
	AType tbl[2];
  };

#ifndef GCC_2_7
template<typename AType> 
  SharedTable<AType> *
  SharedTable<AType>::mySharedTable = NULL;
#endif

template<typename AType> 
  SharedTable<AType>::SharedTable() 
  {
	dataAvailable = -1;
  }

template<typename AType> 
  SharedTable<AType> *
  SharedTable<AType>::instanceSharedTable()
  {
	if (mySharedTable == NULL) {
		mySharedTable = new SharedTable();
	}
	return mySharedTable;
  }

template<typename AType> 
  void
  SharedTable<AType>::writeSharedTable(AType *theTable) 
  {
	if (dataAvailable == 0) {
		tbl[1] = *theTable;
		dataAvailable = 1;
	} else {
		tbl[0] = *theTable;
		dataAvailable = 0;
	}
  }

template<typename AType> 
  BOOL
  SharedTable<AType>::readSharedTable(AType *theTable) 
  {
	if (dataAvailable == 0) {
		*theTable = tbl[0];
	} else if (dataAvailable == 1) {
		*theTable = tbl[1];
	} else {
		return FALSE;
	}
	return TRUE;
  }

template<typename AType> 
  int
  SharedTable<AType>::statSharedTable() 
  {
	return dataAvailable;
  }

#endif
// end of SharedTable.h
