/*!
 *  @file    VxPalibController.h
 *  @brief   VxWorksPaLibrary wrapper
 *           First issue: 1999/04/12 by Toshio HORI
 *
 * Copyright (C) 1999,2000 Toshio HORI.
 */

#ifndef _VXPALIBCONTROLLER_H
#define _VXPALIBCONTROLLER_H

#if 0
/* Include files for VxWorks/Tornado */
#include <vxWorks.h>
#include <stdioLib.h>
#include <stdio.h>
#include <taskLib.h>
#include <sigLib.h>
#include <sockLib.h>
#include <stdlib.h>
#include <ioLib.h>
#include <intLib.h>
#include <kernelLib.h>
#include <msgQLib.h>
#include <selectLib.h>
#include <semLib.h>
#include <strLib.h>
#include <sysLib.h>
#include <netinet/tcp.h>
#include <math.h>
#include <iv.h>
#include <systime.h>
#include <types.h>
#include <socket.h>
#include <in.h>
#include <vme.h>
#endif

#include "Pa10Controller.h"
#include "IfBoard.h"
#include "VxPalib.h"
#include "VxPalibServo.h"
#include "Pa10Param.h"

class VxPalibController : public IfBoard, public Pa10Controller {
public: 
    // Contructors/Destructor (in system.cpp)
    VxPalibController(ARM arm = ARM0);	// Default constructor.
    ~VxPalibController(void);		// Destructor.

    // Public functions.

    // in system.cpp
    PA_ERR initSystem(void);		// Initialize system.
    PA_ERR termSystem(void);		// Terminate system.

    // in motion.cpp
    PA_ERR openArm(void);		// Open arm.
    PA_ERR closeArm(void);	// Close arm.
    PA_ERR startArm(void);	// Start arm control.
    PA_ERR exitArm(void);	// Finish arm control.
    PA_ERR stopArm(SA_WAITSW);		// Stop arm control.
    PA_ERR suspendArm(SA_WAITSW);	// Suspend arm control.
    PA_ERR restartArm(SA_WAITSW);	// Restart arm control.
    PA_ERR execAxis(AXIS, const ANGLE&, SA_WAITSW);	// Exec each axis motion.
    PA_ERR moveMatrix(MOVEMODE, const MATRIX&, const ANGLE&, SA_WAITSW);

    // in direct.C
    PA_ERR setPose(const int&);	// Set arm pose: standing/hanging.

    // in misc.cpp
    PA_ERR resetControl(void);		// Reset control.
    PA_ERR errorMessage(PA_ERR, STRING);	// Print error message.

    // in support.cpp
    PA_ERR modeAxis(void);		// Each axis control mode.
    PA_ERR orderAxis(const ANGLE&);	// Control each axis.
    PA_ERR setTimeout(const int&);	// Set timeout in synch. mode.

    // in status.cpp
    PA_ERR getErrorInfo(int&);		// Get error info.
    PA_ERR getAngle(ANGLE&);		// Get all axis angle.
    PA_ERR getNOA(MATRIX);		// GET (N,O,A,P) matrix.

    // in extstat.cpp
    PA_ERR getMode(int&);		// motion control CPU status.
    PA_ERR getRedundant(JOUMODE&);	// redundant axis control mode in RMRC.
    PA_ERR getVersion(float&);		// motion control program version.
    PA_ERR getTimeout(int&);		// timeout value.

    // in velocity.cpp
    PA_ERR modeVelocity(VELMODE vmode, AXIS axis);

    // in redundant.cpp
    PA_ERR orderVelocity(float speed[]);
    PA_ERR modeRedundant(JOUMODE cmode);
    PA_ERR orderRedundant(JOUTYPE jt);

    // implement IfBoard in dio.cpp
    PA_ERR inputDIO(DIOSTATUS&);
    PA_ERR outputDIO(DIOSTATUS&);
    PA_ERR setDIO(DIOPORT, DIOCH);
    PA_ERR resetDIO(DIOPORT, DIOCH);

    BOOL canMoveMatrix(void);

    // Shared table accessors
    void doSvOn(const int);
    void doSvOff(void);
    void doSvLimitter(const int);
    void putReferenceConfArm(const Frame&);
    void putReferenceVelArm(const float[6]);
    void putReferenceAngArm(const float[7]);
    void putReferenceAngVelArm(const float[7]);
    void setControlCoords(const Frame&);
    void setFGain(const float[6][3]);
    void setComp(const float[6][3]);
    void setJoyMode(const int);
    void setJoyVGain(const float[2]);
    void setJoyFGain(const float[2]);
    void setLimitAbsVelArm(const float[6]);
    void setLimitAngArm(const float[7][2]);
    void setLimitAbsAngVelArm(const float[7]);
    void setLimitAbsFrc(const float[2]);
    void setLimitFrc(const float[6][2]);

    int getSvMode(void);
    int getSvLimitterMode(void);
    void getReferenceConfArm(Frame&);
    void getReferenceVelArm(float[6]);
    void getReferenceAngArm(float[7]);
    void getReferenceAngVelArm(float[7]);
    void getControlCoords(Frame&);
    void getFGain(float[6][3]);
    void getComp(const float[6][3]);
    int getJoyMode(void);
    void getJoyVGain(float[2]);
    void getJoyFGain(float[2]);
    void getLimitAbsVelArm(float[2]);
    void getLimitAngArm(float[7][2]);
    void getLimitAbsAngVelArm(float[7]);
    void getLimitAbsFrc(float[2]);
    void getLimitFrc(float[6][2]);

    int getCounter(void);
    int getLoopTime(void);
    int getExecTime(void);
    int getSvStat(void);
    void getCurrentConfArm(Frame&);
    void getCurrentAngArm(float[7]);
    void getFrc(float[6]);
    void getJoyFrc(float[6]);

    BOOL readServo();
    void writeServo();

private:
    void copyfN(const int, const float*, float*);
    void copyf2(const float[3], float[3]);
    void copyf3(const float[3], float[3]);
    void copyf6(const float[6], float[6]);
    void copyf7(const float[7], float[7]);
    void copyf62(const float[6][2], float[6][2]);
    void copyf63(const float[6][3], float[6][3]);
    void copyf72(const float[7][2], float[7][2]);

    VxPalib * palib;

#ifdef GCC_2_7
    SharedTable_FromServoTable *fromSv;
    SharedTable_ToServoTable *toSv;
#else
    SharedTable<FromServoTable> *fromSv;
    SharedTable<ToServoTable> *toSv;
#endif

    static ToServoTable toSvTbl;
    static FromServoTable fromSvTbl;

}; /* class VxPalibController */

#endif /* _VXPALIBCONTROLLER_H */

/* EOF */
