/*
 * pa_error.h
 * Author   : Toshio HORI <toshi@etl.go.jp>
 * Created  : 1999/04/13
 * Comment  : PA-10 error status.
 *
 * $Id: pa_error.h,v 1.1.1.1 2003/07/14 14:31:49 asahi Exp $
 *
 * Copyright (C) 1999 Toshio HORI.
 */

#ifndef _PA_ERROR_H
#define _PA_ERROR_H

enum ERR {
    /*----- Man-Machine Controller error. -----*/
    ERR_OK		= 0,	// No error.
    ERR_MISMATCH	= -1,	// Argument type mismatch.

    // Action error.
    ERR_FILE		= -1,	// No such file.
    ERR_READ		= -2,	// File read error.
    ERR_WRITE		= -3,	// File write error.
    ERR_INT		= -4,	// Interrupt error.
    ERR_OPEN		= -5,	// Arm not opened.

    /*----- Argument error. -----*/
    ERR_ARM		= -20,	// No such arm.
    ERR_AXIS		= -21,	// No such axis.
    ERR_DRV		= -22,	// No such driver.
    ERR_PB		= -23,	// Playback error.
    ERR_PD		= -24,	// Teaching point deletion error.
    ERR_PA		= -25,	// Teaching point attribute error.
    ERR_PTN		= -26,	// Teaching point velocity pattern error.
    ERR_PT		= -27,	// Teaching point data type error.
    ERR_PM		= -28,	// Teaching point control error.
    ERR_VT		= -29,	// Default velocity change error.
    ERR_VM		= -30,	// Velocity control mode error.
    ERR_JM		= -31,	// Redundant axis control mode error.
    ERR_JT		= -32,	// Redundant axis control error.
    ERR_MM		= -33,	// Tool matrix mode error.
    ERR_DM		= -34,	// Direct control error.
    ERR_DP		= -35,	// Digital I/O port error.
    ERR_DC		= -36,	// Digital I/O channel error.
    ERR_MES		= -37,	// Error code not defined.

    /*----- Warning. -----*/
    ERR_CANT_CPU	= -1000,    // Can't access motion controller.
    ERR_NON_EVNT	= -1001,    // Format/Command mismatch.
    ERR_CANT_EVNT	= -1002,    // Can't handle in current mode.
    ERR_INVALD_EVNT	= -1003,    // Invalid event.

    ERR_S1_VEL		= -1010,    // S1-axis velocity exceeds.
    ERR_S2_VEL		= -1011,    // S2-axis velocity exceeds.
    ERR_S3_VEL		= -1012,    // S3-axis velocity exceeds.
    ERR_E1_VEL		= -1013,    // E1-axis velocity exceeds.
    ERR_E2_VEL		= -1014,    // E2-axis velocity exceeds.
    ERR_W1_VEL		= -1015,    // W1-axis velocity exceeds.
    ERR_W2_VEL		= -1016,    // W2-axis velocity exceeds.

    ERR_XYZ_VEL		= -1018,    // Tool tip position velocity exceeds.
    ERR_YPR_VEL		= -1019,    // Tool tip posture velocity exceeds.

    ERR_S1_SAGL		= -1020,    // S1-axis safety angle exceeds.
    ERR_S2_SAGL		= -1021,    // S2-axis safety angle exceeds.
    ERR_S3_SAGL		= -1022,    // S3-axis safety angle exceeds.
    ERR_E1_SAGL		= -1023,    // E1-axis safety angle exceeds.
    ERR_E2_SAGL		= -1024,    // E2-axis safety angle exceeds.
    ERR_W1_SAGL		= -1025,    // W1-axis safety angle exceeds.
    ERR_W2_SAGL		= -1026,    // W2-axis safety angle exceeds.

    ERR_S1_TAGL		= -1030,    // S1-axis target angle exceeds.
    ERR_S2_TAGL		= -1031,    // S2-axis target angle exceeds.
    ERR_S3_TAGL		= -1032,    // S3-axis target angle exceeds.
    ERR_E1_TAGL		= -1033,    // E1-axis target angle exceeds.
    ERR_E2_TAGL		= -1034,    // E2-axis target angle exceeds.
    ERR_W1_TAGL		= -1035,    // W1-axis target angle exceeds.
    ERR_W2_TAGL		= -1036,    // W2-axis target angle exceeds.

    ERR_LNK_CTL		= -1039,    // Can't continue to keep teaching points.
    ERR_MEM_FULL	= -1040,    // Memory full.
    ERR_MIS_COMD	= -1041,    // Some procedure(s) is/are prerequisite.
    ERR_PB_CIR		= -1042,    // Circle/Arc error.
    ERR_PB_NEXT		= -1043,    // Next pointer doesn't exist.
    ERR_PB_PREV		= -1044,    // Previous pointer doesn't exist.
    ERR_PB_END		= -1045,    // Playback data end.
    ERR_PB_NULL		= -1046,    // No playback datum.
    ERR_PB_REFER	= -1047,    // Playback data reference error.
    ERR_PB_REPLACE	= -1048,    // Playback data replace.
    ERR_PB_PANIC	= -1049,    // Pointer management error.

    ERR_NOT_ENUGH	= -1050,    // Arm length exceeds 895[mm].
    ERR_MIS_PARAM	= -1051,    // Parameter exceeds its limit.
    ERR_NOA_DAT		= -1060,    // Invalid NOA data.
    ERR_PNT_ATR		= -1061,    // Last CP datum retrieved.
    ERR_PTP_DAT		= -1062,    // Exceeds RMRC range.
    ERR_CP_LOGGING	= -1063,    // Currently CP data is logged.
    ERR_FIFO_MAX	= -1064,    // Exceeds max. completion No.
    ERR_FIFO_ARC	= -1065,    // Can't make circle/arc.

    COVERS1		= -1070,    // S1-axis velocity control angle exceeds.
    COVERS2		= -1071,    // S2-axis velocity control angle exceeds.
    COVERS3		= -1072,    // S3-axis velocity control angle exceeds.
    COVERE1		= -1073,    // E1-axis velocity control angle exceeds.
    COVERE2		= -1074,    // E2-axis velocity control angle exceeds.
    COVERW1		= -1075,    // W1-axis velocity control angle exceeds.
    COVERW2		= -1076,    // W2-axis velocity control angle exceeds.

    /*----- Can't continue control. -> Brake lock. -----*/
    ERR_OVER900		= -2017,    // Arm length exceeds 900[mm].

    ERR_S1_AGL		= -2020,    // S1-axis angle exceeds.
    ERR_S2_AGL		= -2021,    // S2-axis angle exceeds.
    ERR_S3_AGL		= -2022,    // S3-axis angle exceeds.
    ERR_E1_AGL		= -2023,    // E1-axis angle exceeds.
    ERR_E2_AGL		= -2024,    // E2-axis angle exceeds.
    ERR_W1_AGL		= -2025,    // W1-axis angle exceeds.
    ERR_W2_AGL		= -2026,    // W2-axis angle exceeds.

    DOVERS1		= -2030,    // S1-axis direct control angle exceeds.
    DOVERS2		= -2031,    // S2-axis direct control angle exceeds.
    DOVERS3		= -2032,    // S3-axis direct control angle exceeds.
    DOVERE1		= -2033,    // E1-axis direct control angle exceeds.
    DOVERE2		= -2034,    // E2-axis direct control angle exceeds.
    DOVERW1		= -2035,    // W1-axis direct control angle exceeds.
    DOVERW2		= -2036,    // W2-axis direct control angle exceeds.

    ERR_CANT_MOVE	= -2051,    // Current arm length exceeds 895[mm].

    ERR_S1_RES		= -2060,    // S1 resolver deviation error.
    ERR_S2_RES		= -2061,    // S2 resolver deviation error.
    ERR_S3_RES		= -2062,    // S3 resolver deviation error.
    ERR_E1_RES		= -2063,    // E1 resolver deviation error.
    ERR_E2_RES		= -2064,    // E2 resolver deviation error.
    ERR_W1_RES		= -2065,    // W1 resolver deviation error.
    ERR_W2_RES		= -2066,    // W2 resolver deviation error.

    ERR_TIMEOUT		= -2070,    // Timeout, automatically stops.
    ERR_SYNCOUT		= -2071,    // Can't get to the destination.

    ERR_SYNC_S1		= -2080,    // S1-axis sync. error in each axis control.
    ERR_SYNC_S2		= -2081,    // S2-axis sync. error in each axis control.
    ERR_SYNC_S3		= -2082,    // S3-axis sync. error in each axis control.
    ERR_SYNC_E1		= -2083,    // E1-axis sync. error in each axis control.
    ERR_SYNC_E2		= -2084,    // E2-axis sync. error in each axis control.
    ERR_SYNC_W1		= -2085,    // W1-axis sync. error in each axis control.
    ERR_SYNC_W2		= -2086,    // W2-axis sync. error in each axis control.

    ERR_RMRC_X		= -2087,    // X-axis sync. error in RMRC control.
    ERR_RMRC_Y		= -2088,    // Y-axis sync. error in RMRC control.
    ERR_RMRC_Z		= -2089,    // Z-axis sync. error in RMRC control.

    ERR_VELOCITY	= -2090,    // Velocity deviation error.
    ERR_RMRC_YPR	= -2091,    // Tip YPR deviation err in RMRC control.

    /*----- Fatal error. -> control stop. -----*/
    ERR_POWER_ON	= -3000,    // Start command not initiated.

    ERR_EM_CTL		= -3001,    // Emergency button is ON.
    ERR_ARC_SEND	= -3002,    // Can't communicate ARCnet.
    ERR_FORCE_ON	= -3010,    // Not in force control mode.

    ERR_D1_B15		= -3020,    // Servo ON/OFF.
    ERR_D1_B14		= -3021,    // Communication error.
    ERR_D1_B13		= -3022,    // Velocity deviation error.
    ERR_D1_B12		= -3023,    // Resolver deviation error.
    ERR_D1_B11		= -3024,    // Velocity limit exceeds.
    ERR_D1_B10		= -3025,    // Torque error.
    ERR_D1_B09		= -3026,    // IPM error.
    ERR_D1_B08		= -3027,    // Brake snap.
    ERR_D1_B07		= -3028,    // Resolver snap.
    ERR_D1_B06		= -3029,    // not used.
    ERR_D1_B05		= -3030,    // not used.
    ERR_D1_B04		= -3031,    // not used.
    ERR_D1_B03		= -3032,    // not used.
    ERR_D1_B02		= -3033,    // not used.
    ERR_D1_B01		= -3034,    // not used.
    ERR_D1_B00		= -3035,    // not used.

    ERR_D2_B15		= -3040,    // Servo ON/OFF.
    ERR_D2_B14		= -3041,    // Communication error.
    ERR_D2_B13		= -3042,    // Velocity deviation error.
    ERR_D2_B12		= -3043,    // Resolver deviation error.
    ERR_D2_B11		= -3044,    // Velocity limit exceeds.
    ERR_D2_B10		= -3045,    // Torque error.
    ERR_D2_B09		= -3046,    // IPM error.
    ERR_D2_B08		= -3047,    // Brake snap.
    ERR_D2_B07		= -3048,    // Resolver snap.
    ERR_D2_B06		= -3049,    // not used.
    ERR_D2_B05		= -3050,    // not used.
    ERR_D2_B04		= -3051,    // not used.
    ERR_D2_B03		= -3052,    // not used.
    ERR_D2_B02		= -3053,    // not used.
    ERR_D2_B01		= -3054,    // not used.
    ERR_D2_B00		= -3055,    // not used.

    ERR_D1_COM		= -3067,    // Communication error with servo driver(1).
    ERR_D2_COM		= -3068,    // Communication error with servo driver(2).
    ERR_D3_COM		= -3069,    // Communication error with servo driver(3).
    ERR_D4_COM		= -3070,    // Communication error with servo driver(4).

    ERR_D11_STA		= -3071,    // Servo driver(S1) status error.
    ERR_D12_STA		= -3072,    // Servo driver(S2) status error.
    ERR_D21_STA		= -3073,    // Servo driver(S3) status error.
    ERR_D22_STA		= -3074,    // Servo driver(E1) status error.
    ERR_D31_STA		= -3075,    // Servo driver(E2) status error.
    ERR_D32_STA		= -3076,    // Servo driver(W1) status error.
    ERR_D41_STA		= -3077,    // Servo driver(W2) status error.

    ERR_S_SUSPD		= -3091,    // Communication start command issue error.
    ERR_E_SUSPD		= -3092,    // Communication end command issue error.
    ERR_I_SUSPD		= -3093,    // Initialize command issue error.

    /*----- System error. -----*/
    ERR_BUS_TIMEOUT	= -4001,    // Local bus timeout error.
    ERR_BOAD_ACFAIL	= -4002,    // Control board AC snap error.
    ERR_WDT_TIMEOUT	= -4004,    // Local bus timeout error.
    ERR_VME_SYSFAIL	= -4008,    // VME bus system snap error.
    ERR_VME_ACFAIL	= -4016,    // VME bus AC snap error.
    ERR_ABT_TIMEOUT	= -4032,    // VME bus arbitration timeout error.
    ERR_VME_TIMEOUT	= -4064	    // VME bus timeout error.
};

#endif /* _PA_ERROR_H */

/* EOF */
