/*
 * patype.h
 * Author   : Toshio HORI <toshi@etl.go.jp>
 * Created  : 1999/04/12
 * Comment  : PA-10 Header file for C++/Tornado environment.
 *
 * $Id: patype.h,v 1.1.1.1 2003/07/14 14:31:49 asahi Exp $
 *
 * Copyright (C) 1999 Toshio HORI.
 */
#ifndef _PA_TYPE_H
#define _PA_TYPE_H

//#define _PA_  10

#if 0
/* Include files for VxWorks/Tornado */
#include <vxWorks.h>
#include <stdioLib.h>
#include <stdio.h>
#include <taskLib.h>
#include <sigLib.h>
#include <sockLib.h>
#include <stdlib.h>
#include <ioLib.h>
#include <intLib.h>
#include <kernelLib.h>
#include <msgQLib.h>
#include <selectLib.h>
#include <semLib.h>
#include <strLib.h>
#include <sysLib.h>
#include <netinet/tcp.h>
#include <math.h>
#include <iv.h>
#include <systime.h>
#include <types.h>
#include <socket.h>
#include <in.h>
#include <vme.h>
#endif

#include <pa_error.h>

/*================================*/
/* Data type definition for PA-10 */
/*================================*/

/*----- Basic data type -----*/
typedef char		BYTE;	/* 8-bit integer */
typedef unsigned char	UBYTE;	/* unsigned 8-bit integer */
typedef float		REAL32;	/* 32-bit floating point */
typedef double		REAL64;	/* 64-bit floating point */
typedef BYTE*		STRING;	/* string */

/*----- special data type -----*/
typedef float	MATRIX[3][4];	/* Matrix */
typedef float	NOAMAT[3][3];	/* NOA matrix */
typedef float	VECTOR[3];	/* Vector */

/*
 * A constant 'ARM' is defined in the header files in Tornado 2.0 for
 * StrongARM processors.  We don't want it to be defined so we 'undef'ine it.
 */
#ifdef ARM
#undef ARM
#endif

/*----- Arm numbers -----*/
enum ARM {
    ARM0 = 0, ARM1, ARM2, ARM3, ARM4, ARM5, ARM6, ARM7,
    ARM8, ARM9, ARM10, ARM11, ARM12, ARM13, ARM14, ARM15
};

/*----- Data copy direction -----*/
enum COPY_DIR {
    GET = 1, PUT = 2, COPY = 3
};

/*----- Function data type -----*/
typedef ERR	    (*WFP)(int);
#define WM_WAIT	    ((WFP)0)
#define WM_NOWAIT   ((WFP)1)

/*----- Teaching data -----*/
struct PNTDAT {
    int	    s1;
    int	    s2;
    int	    s3;
    int	    e1;
    int	    e2;
    int	    w1;
    int	    w2;
    int	    speed;
    int	    rotate;
    int	    type;
    int	    fifo;
    int	    pattern;
    int	    time;
    int	    number;
    int	    tool0;
    int	    tool1;
};
typedef PNTDAT*	    PNTDATP;

/*----- Joint status -----*/
struct ANGLE {
    float	s1;
    float	s2;
    float	s3;
    float	e1;
    float	e2;
    float	w1;
    float	w2;
};
typedef ANGLE*	    ANGLEP;

/*----- Arm status -----*/
struct ARMSTATUS {
    int		remote;
    int		count;
    int		error;
    ANGLE	angle;
    MATRIX	noap;
    float	ypr[3];
};
typedef ARMSTATUS*	ARMSTATUSP;

/*----- Digital I/O data structure -----*/
struct DIOSTATUS {
    unsigned char   io0;	// DIO 1ch
    unsigned char   io1;	// DIO 2ch
    unsigned char   io2;	// DIO 3ch
    unsigned char   io3;	// DIO 4ch
};
typedef DIOSTATUS*	DIOSTATUSP;

/*----- Parameters -----*/
struct PARAM {
    float	resl;		// Resolver resolusion
    int		pul[7];		// Position limitter(+)
    int		pdl[7];		// Position limitter(-)
    int		vel[7+2];	// Velocity limitter
    int		dev[7+2];	// Default velocity
    int		pg1[7];		// Position control gain 1
    int		pg2[7];		// Position control gain 2
    int		vg1[7];		// Velocity control gain
    int		tg1[7];		// Torque control gain
    int		pcm[7];		// Position control selection matrix
    int		fcm[7];		// Force control selection matrix
    int		arl[7];		// Arm length
    int		arg[7];		// Center of gravity of arm
    int		arw[7];		// Arm weight
    int		rfp[7];		// Reference of home position
    int		rsp[7];		// Reference of safety position
    int		rop[7];		// Reference of other position
    int		tol[7];		// Tool parameter
    int		dmy[7];		// Waving parameter
    int		spa[7];		// Spare slot
};
typedef PARAM*		PARAMP;

/*----- Enumerate each axis -----*/
enum AXIS {
    S1 = 0x01,
    S2 = 0x02,
    S3 = 0x04,
    E1 = 0x08,
    E2 = 0x10,
    W1 = 0x20,
    W2 = 0x40
};

const int	AXISALL		= (S1|S2|S3|E1|E2|W1|W2);	// = 0x7f
const int	ALLAXIS		= AXISALL;
const int	LOCKAXIS_S1	= (S2|S3|E1|E2|W1|W2);		// = 0x7e
const int	LOCKAXIS_S3	= (S1|S2|E1|E2|W1|W2);		// = 0x7b

/*----- Servo drivers -----*/
enum DRIVER {
    DRV1 = 0, DRV2, DRV3, DRV4
};

/*----- Playback mode -----*/
enum PLAYBACK {
    PB_FORES,
    PB_BACKS,
    PB_FORE,
    PB_BACK
};

/*----- Teaching points deletion -----*/
enum PNTDEL {
    PD_CUR = 0x7500,	// Delete current teaching point
    PD_FORE,		// Delete previous teaching point
    PD_ALL		// Delete all teaching points
};

/*----- Teaching point attribute indicators -----*/
enum PNTATTR {
    PA_CHGVEL = 0x7300,
    PA_CHGWAIT,
    PA_VELPTN,		// Teaching data: velocity pattern
    PA_ROTVEL		// Teaching data: rotational velocity
};

/*----- Teaching point types -----*/
enum PNTTYPE {
    PT_CP = 0x7100,
    PT_PTP,
    PT_BCP,
    PT_BPTP,
    PT_ARC1, PT_ARC2, PT_ARC3,
    PT_CIR1, PT_CIR2, PT_CIR3,
    PT_AXS, PT_BAXS
};

/*----- Manipulation of pointers to teaching points -----*/
enum PNTMOVE {
    PM_TOP = 0x7100,	// Go to the first point
    PM_NEXT,		// Go to the next point
    PM_PREV,		// Go to the previous point
    PM_BTM,		// Go to the last point
    PM_JMP,		// Go to the designated point
    PM_CIR,		// Go to the first teaching point on a circle
    PM_ARC		// Go to the first teaching point on an arc
};

/*----- Default velocity modification type -----*/
enum VELTYPE {
    VT_ONEVEL,		// Modify default velocity of each axis
    VT_XYZVEL,		// Modify default velocity of tip position
    VT_YPRVEL		// Modify default velocity of tip posture
};

/*----- Velocity control mode -----*/
enum VELMODE {
    VM_XYZ = 0x200,
    VM_YPR,
    VM_xyz,
    VM_ypr,
    VM_ONE,
    VM_XYZYPR,
    VM_xyzypr
};

/*----- Redundant axis control mode -----*/
enum JOUMODE {
    JM_SET	= 0x345,
    JM_RESET	= 0x346,
    JM_VSET,
    JM_ON,
    JM_OFF,
    JM_S3ON,
    JM_S3DIV,
    JM_S3HOLD
};

/*----- Redundant axis control type -----*/
enum JOUTYPE {
    JT_RIGHT	= 1,
    JT_HOLD	= 0,
    JT_LEFT	= -1
};

/*----- Motion mode -----*/
enum MOVEMODE {
    MM_XYZ	= 0x5680,
    MM_NOA,
    MM_XYZNOA
};

/*----- Direct control mode -----*/
enum DIRECTMODE {
    DM_STOP,
    DM_START
};

/*----- Arm position -----*/
const int ARM_STANDING	=  1;	// Located on the floor
const int ARM_HANGING	= -1;	// Hang on the ceiling

/*----- Digital I/O port number -----*/
enum DIOPORT {
    DP_PORT0	=  0,
    DP_PORT1	=  1,
    DP_PORT2	=  2,
    DP_PORT3	=  3,
    DPO_PORT0	=  4,
    DPO_PORT1	=  5,
    DPO_PORT2	=  6,
    DPO_PORT3	=  7,
    DPX_PORT0	=  8,
    DPX_PORT1	=  9,
    DPX_PORT2	= 10,
    DPX_PORT3	= 11
};

/*----- Digital I/O channel number -----*/
enum DIOCH {
    DC_CH1	= 0,
    DC_CH2	= 1,
    DC_CH3	= 2,
    DC_CH4	= 3,
    DC_CH5	= 4,
    DC_CH6	= 5,
    DC_CH7	= 6,
    DC_CH8	= 7
};

/*----- Waving and sensor calibration coordinate type -----*/
enum TRANSMODE {
    MODE_xyz	= 0x01,
    MODEIxyz	= 0x02,
    MODE_XYZ	= 0x04,
    MODEIXYZ	= 0x08,
    MODE_wave	= 0x10,
    MODEIwave	= 0x20
};

/*----- Teaching points attribute -----*/
enum PNTID {
    PA_SETID	= 0x7304
};

/*----- Teaching points' number for a circle/an arc -----*/
enum PNTNO {
    PN_1	= 1,
    PN_2	= 2,
    PN_3	= 3
};

/*----- Waving and sensor coordination data structure -----*/
struct TRANSMAT {
    int		Enable;		// Enabler
    float	_xyz[3];	// Hand coordinate, without integral offset
    float	Ixyz[3];	// Hand coordinate, with integral offset
    float	_XYZ[3];	// Robot coordinate, without integral offset
    float	IXYZ[3];	// Robot coordinate, with integral offset
    float	_wave[3];	// Path coordinate, without integral offset
    float	Iwave[3];	// Path coordinate, with integral offset
};
typedef TRANSMAT*	TRANSMATP;

/*----- Waving parameters' attribute -----*/
struct WAVATTR {
    int		limit;		// Waving width limit
    int		pitch;		// Waving pitch
    int		depth;		// Waving depth
    int		width;		// Waving width
    float	angle;		// Waving angle
};

/*----- Arm target structure -----*/
struct ARMTARGET {
    ANGLE	angle;
    MATRIX	noap;
    float	ypr[3];
};
typedef ARMTARGET*	ARMTARGETP;

#endif /* _PA_TYPE_H */

/* EOF */
