#ifndef __INCVxPalibServo_h
#define __INCVxPalibServo_h
/*!
*   @file  VxPalibServo.h 
*   @brief servo header 
*
*   Copyright 1999-2003 Kosei Kitagaki 
*/
/*
modification history
--------------------
01a,24feb03,kk  written
*/

#if (__GNUC__ == 2) && (__GNUC_MINOR__ == 7)
#define GCC_2_7
#endif

#include "DataType.h"
#include "patype.h"
#include "Pa10Controller.h"
#include "ExFtSensor.h"
#include "FtsJoy.h"
#ifndef GCC_2_7
#include "SharedTable.h"
#else
#include "VxSharedTable.h"
#endif
#include "ServoArmPack.h"
#include "AngleLimitter.h"
#include "Pa10Param.h"
//#include "CMyTimer.h"

#define bitCheck(x,type) (((x)&(type))==(type))

class VxPalibServo
{
public:
    void init(float theLoopTimeBySec, ArmPack*);
    void servo();

private:
    void svStop(const int arm);
    void svRestart(const int arm);

    int velocityControl(const int arm, FLOAT6 vel);
    int angleVelocityControl(const int arm, FLOAT7 angvel);

    BOOL velocityLimitter(FLOAT6 v, const FLOAT6 vlimit);
    BOOL angleVelocityLimitter(FLOAT7 angv, const FLOAT7 angvlimit);

    void calcCompControl(const Frame& p_conf, const Frame& r_conf, 
		const DoubleVec& e_frc, const float fComp[6][3], FLOAT6& vel);
    void calcForceControl(const Frame& p_conf, const DoubleVec& r_frc, 
		const DoubleVec& e_frc, const float fGain[6][3], FLOAT6& vel);
    void calcJoyMasterControl(const int arm, const int mode,
		const DoubleVec& e_frc, 
		const DoubleVec& j_frc, const float jGain[2], FLOAT6& vel);

    void clear6(FLOAT6&);
    void clear7(FLOAT7&);
    void add6(const FLOAT6&, FLOAT6&);
    void add7(const FLOAT7&, FLOAT7&);
    void copyf6(const FLOAT6&, FLOAT6&);
    void copyf7(const FLOAT7&, FLOAT7&);

#ifdef GCC_2_7
    static SharedTable_FromServoTable* fromSv[ARM_MAX_NUM];
    static SharedTable_ToServoTable* toSv[ARM_MAX_NUM];
#else
    static SharedTable<FromServoTable>* fromSv[ARM_MAX_NUM];
    static SharedTable<ToServoTable>* toSv[ARM_MAX_NUM];
#endif
    static ToServoTable toSvTbl[ARM_MAX_NUM];
    static FromServoTable fromSvTbl[ARM_MAX_NUM];
    
    static Pa10* myPa10[ARM_MAX_NUM];
    static Pa10Controller* myPa10Controller[ARM_MAX_NUM];
    static ExFtSensor* myExFtSensor[ARM_MAX_NUM];
    static FtsJoy* myFtsJoy[ARM_MAX_NUM];
    static AngleLimitter* myAngleLimitter[ARM_MAX_NUM];
    
    static int armNum;	
    static int counter;
    static float loopTimeBySec;
};

#endif
// end of VxPalibServo.h
