/*
 * pactl++.h	: PA-10 control data structures.
 * Author	: Toshio HORI <toshi@etl.go.jp>
 * Created	: 1999/04/23
 * Comment	: Original 'C' version was written by MHI.
 *
 * $Id: pactl++.h,v 1.1.1.1 2003/04/09 10:38:26 asahi Exp $
 *
 * Copyright (C) 1999 Toshio HORI.
 */

// K.Kitagaki 05nov02 ---
#ifdef DEBUG
#undef DEBUG
#endif
// ---

#ifndef _PA_CTLPP_H
#define _PA_CTLPP_H

const int   EPDATA		= 16;
const int   AXIS_NUMBER		= 7;
const int   VECT_NUMBER		= 6;
const int   COORD_NUMBER	= 3;
const int   DATABLOCK_NUMBER	= 5;
const int   ARCNETPACET		= 256;

/*----- Communication header description -----*/
struct HEADER {
    int	    ComMode;	    // Communication mode
    int	    Event;	    // Event
    int	    Status;	    // Status
    int	    CtrlMode[4];    // Motion control mode
    int	    Dummy[8];	    // Spare slots
};

/*----- High-lank controller communicationi structure (UP) -----*/
struct DOWN {
    int		Axis;			// Designate axis
    float	Velocity[AXIS_NUMBER];	// Goal velocity
    float	Angle[AXIS_NUMBER];	// Goal angle
    float	Position[COORD_NUMBER];	// Goal position
    float	Posture[COORD_NUMBER];	// Goal posture
    int		Vdata;			// Playback velocity
};

struct LDOWN {
    int		Axis;			// Designate axis
    int		Velocity[AXIS_NUMBER];	// Goal velocity
    int		Angle[AXIS_NUMBER];	// Goal angle
    int		Position[COORD_NUMBER];	// Goal position
    int		Posture[COORD_NUMBER];	// Goal posture
    int		Vdata;			// Playback velocity
};

/*----- Teaching data structure parts -----*/
struct TEACH {
    int	    BlockNum;
    int	    BlockCount;
    int	    TeachData[DATABLOCK_NUMBER][EPDATA];
};

/*----- Robot output data structure -----*/
struct OUTPUT {
    int	    MBrake;		    // Brake ON/OFF
    int	    GateBlock;		    // Servo ON/OFF
    int	    VTSwitch;		    // Velocity/Torque switch
    int	    Torque[AXIS_NUMBER];    // Torque commands
    int	    Velocity[AXIS_NUMBER];  // Velocity commands
    int	    Dummy[7];		    // Spare slots
};

/*----- Robot input data structure -----*/
struct INPUT {
    int	    Rez2[AXIS_NUMBER];		// Resolver
    int	    Torque[AXIS_NUMBER];	// Torque
    int	    Status[AXIS_NUMBER*2];	// Servo status
    int	    Dummy[AXIS_NUMBER];		// Spare slots
};

/*----- Target [N,O,A,P] structure -----*/
struct TARGET {
    int		each[AXIS_NUMBER];		    // Goal angle/velocity
    float	xyz[COORD_NUMBER];		    // Goal position
    float	noa[COORD_NUMBER][COORD_NUMBER];    // Goal posture
    float	force[VECT_NUMBER];		    // Goal force
};

struct LTARGET {
    int	    each[AXIS_NUMBER];			// Goal angle/velocity
    int	    xyz[COORD_NUMBER];			// Goal position
    int	    noa[COORD_NUMBER][COORD_NUMBER];	// Goal posture
    int	    force[VECT_NUMBER];			// Goal force
};

/*----- Mark [N,O,A,P] structure -----*/
struct MARK {
    int		each[AXIS_NUMBER];		    // Current angle/velocity
    float	xyz[COORD_NUMBER];		    // Current position
    float	noa[COORD_NUMBER][COORD_NUMBER];    // Current posture
    float	force[VECT_NUMBER];		    // Current force
};

/*----- LMark [N,O,A,P] structure -----*/
struct LMARK {
    int	    each[AXIS_NUMBER];			// Current angle/velocity
    int	    xyz[COORD_NUMBER];			// Current position
    int	    noa[COORD_NUMBER][COORD_NUMBER];	// Current posture
    int	    force[VECT_NUMBER];			// Current force
};

struct FLOATMARK {
    float	each[AXIS_NUMBER];		    // Current angle/velocity
    float	xyz[COORD_NUMBER];		    // Current position
    float	noa[COORD_NUMBER][COORD_NUMBER];    // Current posture
    float	force[VECT_NUMBER];		    // Current force
};

/*----- Double mark -----*/
struct DMARK {
    int		each[AXIS_NUMBER];		    // Current angle/velocity
    double	xyz[COORD_NUMBER];		    // Current position
    double	noa[COORD_NUMBER][COORD_NUMBER];    // Current posture
    double	force[VECT_NUMBER];		    // Current force
};

/*----- Jacobian inverse structure -----*/
struct JACOB {
    float	jac[AXIS_NUMBER][AXIS_NUMBER];	    // Jacobian
};

struct VERJAC {
    float	ver_jac[AXIS_NUMBER][AXIS_NUMBER];  // Pseudo Jacobian
};

struct JAC {
    JACOB	_jac;
    VERJAC	_ver_jac;
};

struct LJAC {
    int	    jac[AXIS_NUMBER][AXIS_NUMBER];	// Jacobian
    int	    ver_jac[AXIS_NUMBER][AXIS_NUMBER];	// Pseudo Jacobian
};

/*----- Error note structure -----*/
struct ERRORNOTE {
    int	    Warning[AXIS_NUMBER];
    int	    Error[AXIS_NUMBER];
    int	    dummy[16];
};

/*----- Parameter structure -----*/
struct PARAMETER {
    int	    PUL[AXIS_NUMBER];		// Position limitter(+)
    int	    PDL[AXIS_NUMBER];		// Position limitter(-)
    int	    VEL[AXIS_NUMBER + 2];	// Velocity limitter
    int	    DEV[AXIS_NUMBER + 2];	// Default velocity
    int	    PG1[AXIS_NUMBER];		// Position control gain 1
    int	    PG2[AXIS_NUMBER];		// Position control gain 2
    int	    VG1[AXIS_NUMBER];		// Velocity control gain
    int	    TG1[AXIS_NUMBER];		// Force control gain
    int	    PCM[AXIS_NUMBER];		// Position control selection matrix
    int	    FCM[AXIS_NUMBER];		// Force control selection matrix
    int	    ARL[AXIS_NUMBER];		// The length of each axis
    int	    ARG[AXIS_NUMBER];		// Center of gravity of each axis
    int	    ARW[AXIS_NUMBER];		// The weight of each axis
    int	    RFP[AXIS_NUMBER];		// Reference of home position
    int	    RSP[AXIS_NUMBER];		// Reference of safety position
    int	    ROP[AXIS_NUMBER];		// Reference of other position
    int	    TOL[AXIS_NUMBER];		// Tool parameters
    int	    DMY[AXIS_NUMBER];		// Waving parameters
    int	    SPA[AXIS_NUMBER];		// Spare slots
};

/*----- ARC-NET transform structure -----*/
struct ARCNETBASE {
    unsigned char	packet[256];	// Servo driver communication packet
};

/*----- DSP communication flag -----*/
struct FLAG {
    int	    SEND[3];	// DSP action command
    int	    RECV;	// Reply from DSP
};

/*----- Standard digital I/O -----*/
struct DIO {
    unsigned short int	i1420[2];	// Digital input
    unsigned short int	o1420[2];	// Digital output
    unsigned short int	x1420[2];	// Digital output (Ver.1.3)
};

/*----- Debug monitor -----*/
struct DEBUG {
    int		L[14];	// Long data debug area
    float	F[14];
};

/*----- 386 - 486 communication flag -----*/
struct FOR386 {
    int	    flag[7];	// 386 communication flag
};

/*----- x,y,z,y,p,r -----*/
struct XYZYPR {
    float	Meach[7];   // Current angle[rad] of each axis
    float	Teach[7];   // Goal angle[rad] of each axis
    float	Mxyzypr[6]; // Current position/posture in robot-coordinates
    float	Txyzypr[6]; // Goal position/posture in robot-coordinates
    float	Mlen[7];    // Distance between S2-axis and W2-axis
};

/*----- Arm constant data -----*/
struct ARMC {
    float	RTV[7];	// Resolver transform coefficient
    float	LEN[8];	// [0]-[6]: Arm length, [7]: RMRC control limit length
    float	SPA[7];	// [0]: control synch., [1]-[6]: spare slots.
    PNTDAT	PNT;	// Teaching data
    PNTDAT	PNT2;	// Teaching data(realtime set) (Ver.1.3)
    int		RLF;	// remote: 1, local: 0
    int		EMF;	// Emergency stop ON: 1, OFF: 0
};

/*----- Error logging format -----*/
struct CALDATE {
    unsigned char	year;
    unsigned char	mon;
    unsigned char	day;
    unsigned char	hour;
    unsigned char	min;
    unsigned char	sec;
};

struct ERRLOG {
    int		    err;
    CALDATE	    date;
    unsigned char   _date[2];
};

struct LOGDAT {
    CALDATE		ontime;		// Set by Power ON
    unsigned char	_ontime[2];
    CALDATE		offtime;	// Set by Power OFF
    unsigned char	_offtime[2];
    CALDATE		start;		// Set by "PowerOnEvent"
    unsigned char	_start[2];
    CALDATE		term;		// Set by "PowerOffEvent"
    unsigned char	_term[2];
    unsigned int	rtime[2];	// Set on timer task
    unsigned char	top;		// Set at _error_notice function
    unsigned char	_top[3];
    ERRLOG		logdat[256];
};

/*===== Communication format =====*/

/*----- Command area -----*/
struct HIGHDOWN {
    DOWN	Down;	    // Command data;
    OUTPUT	Output;	    // I/O check data
    TEACH	Teach;	    // Teaching data
    ARCNETBASE	Base;	    // Data for servo driver
    PARAMETER	Prm;	    // Parameter table
    MARK	Work[2];    // Transform matrices
    FLOATMARK	TMAT;	    // J/Sprm
};

struct HD {
    HEADER	Header;
    HIGHDOWN	HighDown;
};

/*----- Answer area -----*/
struct ROBOTIO {	// Robot I/O data
    INPUT	Input;
    OUTPUT	Output;
};

struct NOAPDATA {	// Current values & Goal values
    MARK	Mark;
    TARGET	Target;
};

struct HIGHUP {
    HEADER	Header;
    ROBOTIO	RobotIO;	// Robot communication data
    NOAPDATA	NoapData;	// Current values & Goal values
    JACOB	Jacob;		// Jacobian
    TEACH	Teach;		// Teaching data
    ARCNETBASE	Base;		// Data for servo driver
    ERRORNOTE	ErrorNote;	// Error message
    DIO		Dio;		// Digital I/O
    FOR386	For386;		// For 386
    XYZYPR	Xyzypr;		// Current/Goal of arm (Man-Machine controller)
    DEBUG	Debug;		// Debug data
    LOGDAT	LogDat;
    ARMC	Armc;
};

/*===== Communication data for DSP =====*/

/*----- Normal DSP input format -----*/
struct DSP_IN {
    HEADER	Header;
    DOWN	Down;
    INPUT	Input;
    TARGET	Target;
    TARGET	FTarget;
    MARK	Work[2];
};

/*----- IEEE transfer format -----*/
struct LDSP_IN {
    HEADER	Header;
    LDOWN	Down;
    INPUT	Input;
    LTARGET	Target;
    LTARGET	FTarget;
    LMARK	Work[2];
};

union DI {
    DSP_IN	DspInn;
    LDSP_IN	LDspInn;
};

/*----- Normal DSP output format -----*/
struct DSP_OUT {
    HEADER	Header;
    OUTPUT	Output;
    MARK	Mark;
    JAC		Jac;
    ERRORNOTE	ErrorNote;
};

/*----- IEEE transfer format -----*/
struct LDSP_OUT {
    HEADER	Header;
    OUTPUT	Output;
    LMARK	Mark;
    LJAC	Jac;
    ERRORNOTE	ErrorNote;
};

union DO {
    DSP_OUT	DspOut;
    LDSP_OUT	LDspOut;
};

/*===== Structures added in Ver.1.3 =====*/

/*----- Force control parameters -----*/
struct D_FPARAMETER {
    float	SNS[6];
    float	TOG[6];
    float	XCE[6][6];
    float	XCI[6][6];
    float	VEC[3][3];
    float	MU1[6];
    float	MU2[6];
    float	MD1[6];
    float	MD2[6];
    float	FGR[6];
    float	FGH[6];
    float	V2N[6];
    float	DEV[6];
    float	SP1[6];
    float	SP2[6];
    float	DMY[6];
};

/*----- Force control commands -----*/
struct D_PFMAT {
    float	PMX[6];
    float	FMX[6];
};

struct D_FDOWN {
    int		Coord;
    float	Force[6];
    int		Fdata;
};

struct D_FORCE {
    float	ORGforce[6];
    float	OFFforce[6];
    float	XCEforce[6];
    float	TGTforce[6];
    float	XCIforce[6];
};

/*----- Communication map -----*/
/* common space for man-machine controller(MMC) and motion controller(MC) */
struct COMM_DATA {
    DI		    dspi;	// DSP input
    DO		    dspo;	// DSP output
    PARAMETER	    _PRM;	// motion controller parameter
    FLAG	    flag;	// DSP command line from 486
    HD		    hd;		// For communication (MMC  -> MC)
    HIGHUP	    hu;		// For communication (MMC <-  MC)
    TRANSMAT	    trans;	// Waving, sensor, etc. data
    D_FPARAMETER    fprm;	// Force control parameter
    D_FDOWN	    fdown;	// Force control command
    D_PFMAT	    pfmat;	// Position/Force selection matrix
    D_FORCE	    force;	// Force sensor input
    DEBUG	    debug;	// For debug
    JAC		    Jac;	// Jacobian
    MARK	    mat0;	// Transform matrix 0 (work)
    MARK	    mat1;	// Transform matrix 1 (teaching data)
    TRANSMAT	    mtrans;	// Waving, sensor, etc. addition data
};
typedef COMM_DATA*	COMMptr;

#endif /* _PA_CTLPP_H */

/* EOF */
