/*
 * pammc++.h	: For PA-10 Man-machine controller
 * Author	: Toshio HORI <toshi@etl.go.jp>
 * Created	: 1999/04/23
 * Comment	: Original 'C' version was written by MHI.
 *
 * $Id: pammc++.h,v 1.1.1.1 2003/04/09 10:38:26 asahi Exp $
 *
 * Copyright (C) 1999 Toshio HORI.
 */

#ifndef _PA_MMCPP_H
#define _PA_MMCPP_H

#include <pactl++.h>

/*----- Servo driver parameter input structure -----*/
struct MechaZero {
    unsigned char	stat;	// set, def, reset
    int			angle0;	// 1st-axis angle
    int			angle1;	// 2nd-axis angle
};
typedef MechaZero*	MechaZeroPtr;

struct MechaZeroDSP {
    MechaZero	dsp0;
    MechaZero	dsp1;
    MechaZero	dsp2;
    MechaZero	dsp3;
};
typedef MechaZeroDSP*	MechaZeroDSPPtr;

struct PIGain { // Velocity PI gain
    short	pGain;
    short	iGain;
};
typedef PIGain*		PIGainPtr;

struct SpeedPIGain {
    PIGain	axis0;
    PIGain	axis1;
};
typedef SpeedPIGain*	SpeedPIGainPtr;

struct SpeedPIGainDSP {
    SpeedPIGain		dsp0;
    SpeedPIGain		dsp1;
    SpeedPIGain		dsp2;
    SpeedPIGain		dsp3;
};
typedef SpeedPIGainDSP*		SpeedPIGainDSPPtr;

struct PosLmt {
    int	    plus;
    int	    minus;
};
typedef PosLmt*		PosLmtPtr;

struct MotorPosLmt { // Motor limit
    PosLmt	angle0;
    PosLmt	angle1;
};
typedef MotorPosLmt*	MotorPosLmtPtr;

struct MotorPosLmtDSP {
    MotorPosLmt	    dsp0;
    MotorPosLmt	    dsp1;
    MotorPosLmt	    dsp2;
    MotorPosLmt	    dsp3;
};
typedef MotorPosLmtDSP*		MotorPosLmtDSPPtr;

struct ResoOffset { // Resolver offset
    short	res_off;
};
typedef ResoOffset*		ResoOffsetPtr;

struct ResoOffsets { // Resolver offset (two axes)
    ResoOffset	angle0;
    ResoOffset	angle1;
};
typedef ResoOffsets*	ResoOffsetsPtr;

struct ResoOffsetDSP {
    ResoOffsets	    dsp0;
    ResoOffsets	    dsp1;
    ResoOffsets	    dsp2;
    ResoOffsets	    dsp3;
};
typedef ResoOffsetDSP*		ResoOffsetDSPPtr;

struct Reserve { // DSP parameter spare slots.
    int	    reserve[7];
};
typedef Reserve*	ReservePtr;

enum FileID { // File discriminator
    _DRV_PRM	= 0x0729,
    _CTL_PRM	= 0x0512,
    _PNT_DAT	= 0x1965
};

#define COMMENTMAX	38
#define	MAXPATH1	12

struct File {
    FileID  id;				// File discriminator
    BYTE    path[MAXPATH1 + 1];		// File name
    BYTE    comment[COMMENTMAX + 1];	// Comment
};
typedef File*	FilePtr;

/*----- Driver parameter -----*/
struct DspPara {
    MechaZeroDSP	mecha_zero_dsp;		// Mechanical zero-point
    SpeedPIGainDSP	speed_PI_gain_dsp;	// Velocity PI gain
    MotorPosLmtDSP	motor_pos_lmt_dsp;	// Motor-side limit
    ResoOffsetDSP	reso_offset_dsp;	// Resolver offset
    Reserve		reserve;		// DSP parameter spare slot
};
typedef DspPara*	DspParaPtr;

typedef PNTDAT		Record;
typedef PNTDATP		RecordPtr;

const int   RETRY_COUNT	= 10;	// Interrupt retry counts
const int   MAX_ARM 	= 16;	// Maximum arm no.

#define waitFunc(wfp) \
    if (wfp != WM_NOWAIT) { \
	if (wfp != WM_WAIT) { return wfp(status);  } \
	else                { while (isFinish()) ; } \
    }

#endif /* _PA_MMCPP_H */

/* EOF */
