// -*- C++ -*-
/*!
 * @file  slider.cpp
 * @brief Slider component
 * $Date$
 *
 * $Id$
 */

#include "slider.h"

// Module specification
static const char* slider_spec[] =
  {
    "implementation_id", "slider",
    "type_name",         "DataFlowComponent",
    "description",       "Slider component",
    "version",           "1.0",
    "vendor",            "S.Kurihara",
    "category",          "gui",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };


slider::slider(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    m_GainOut("Gain", m_Gain),
    dummy(0)
{
  m_Gain.data.length(d_size);

  // Set OutPort buffer
  registerOutPort("Gain", m_GainOut);
}


slider::~slider()
{
}


RTC::ReturnCode_t slider::onActivated(RTC::UniqueId ec_id)
{
  m_Gain.data.length(d_size);
  for (int i = 0; i < d_size; i++) {
    m_Gain.data[i] = 0.0;
  }

  return RTC::OK;
}


RTC::ReturnCode_t slider::onExecute(RTC::UniqueId ec_id)
{
  m_GainOut << m_Gain;

  return RTC::OK;
}


void slider::set_data(std::vector<float> data)
{
  int data_size = (int)(data.size());

  if (data_size == d_size) {
    for (int i = 0; i < d_size; i++) {
      m_Gain.data[i] = data[i];
    }
  }
}


extern "C"
{
 
  void sliderInit(RTC::Manager* manager)
  {
    RTC::Properties profile(slider_spec);
    manager->registerFactory(profile,
                             RTC::Create<slider>,
                             RTC::Delete<slider>);
  }
  
};


